@@ -9,6 +9,7 @@
 #include "naev.h"
 /** @endcond */
 
+#include "lualib.h"
 #include "physfs.h"
 
 #include "conf.h"
@@ -25,51 +26,51 @@
 #include "space.h"
 #include "utf8.h"
 
-#define conf_loadInt( env, n, i )                                              \
+#define conf_loadInt( L, n, i )                                                \
    {                                                                           \
-      nlua_getenv( naevL, env, n );                                            \
-      if ( lua_isnumber( naevL, -1 ) ) {                                       \
-         i = (int)lua_tonumber( naevL, -1 );                                   \
+      lua_getglobal( L, n );                                                   \
+      if ( lua_isnumber( L, -1 ) ) {                                           \
+         i = lua_tointeger( L, -1 );                                           \
       }                                                                        \
-      lua_pop( naevL, 1 );                                                     \
+      lua_pop( L, 1 );                                                         \
    }
 
-#define conf_loadFloat( env, n, f )                                            \
+#define conf_loadFloat( L, n, f )                                              \
    {                                                                           \
-      nlua_getenv( naevL, env, n );                                            \
-      if ( lua_isnumber( naevL, -1 ) ) {                                       \
-         f = (double)lua_tonumber( naevL, -1 );                                \
+      lua_getglobal( L, n );                                                   \
+      if ( lua_isnumber( L, -1 ) ) {                                           \
+         f = lua_tonumber( L, -1 );                                            \
       }                                                                        \
-      lua_pop( naevL, 1 );                                                     \
+      lua_pop( L, 1 );                                                         \
    }
 
-#define conf_loadTime( env, n, i )                                             \
+#define conf_loadTime( L, n, i )                                               \
    {                                                                           \
-      nlua_getenv( naevL, env, n );                                            \
-      if ( lua_isnumber( naevL, -1 ) ) {                                       \
-         i = (time_t)lua_tonumber( naevL, -1 );                                \
+      lua_getglobal( L, n );                                                   \
+      if ( lua_isnumber( L, -1 ) ) {                                           \
+         i = (time_t)lua_tonumber( L, -1 );                                    \
       }                                                                        \
-      lua_pop( naevL, 1 );                                                     \
+      lua_pop( L, 1 );                                                         \
    }
 
-#define conf_loadBool( env, n, b )                                             \
+#define conf_loadBool( L, n, b )                                               \
    {                                                                           \
-      nlua_getenv( naevL, env, n );                                            \
-      if ( lua_isnumber( naevL, -1 ) )                                         \
-         b = ( lua_tonumber( naevL, -1 ) != 0. );                              \
-      else if ( !lua_isnil( naevL, -1 ) )                                      \
-         b = lua_toboolean( naevL, -1 );                                       \
-      lua_pop( naevL, 1 );                                                     \
+      lua_getglobal( L, n );                                                   \
+      if ( lua_isnumber( L, -1 ) )                                             \
+         b = ( lua_tonumber( L, -1 ) != 0. );                                  \
+      else if ( !lua_isnil( L, -1 ) )                                          \
+         b = lua_toboolean( L, -1 );                                           \
+      lua_pop( L, 1 );                                                         \
    }
 
-#define conf_loadString( env, n, s )                                           \
+#define conf_loadString( L, n, s )                                             \
    {                                                                           \
-      nlua_getenv( naevL, env, n );                                            \
-      if ( lua_isstring( naevL, -1 ) ) {                                       \
+      lua_getglobal( L, n );                                                   \
+      if ( lua_isstring( L, -1 ) ) {                                           \
          free( s );                                                            \
-         s = strdup( lua_tostring( naevL, -1 ) );                              \
+         s = strdup( lua_tostring( L, -1 ) );                                  \
       }                                                                        \
-      lua_pop( naevL, 1 );                                                     \
+      lua_pop( L, 1 );                                                         \
    }
 
 /* Global configuration. */
@@ -129,6 +130,7 @@ void conf_setDefaults( void )
    conf.map_overlay_opacity = MAP_OVERLAY_OPACITY_DEFAULT;
    conf.big_icons           = BIG_ICONS_DEFAULT;
    conf.always_radar        = 0;
+   conf.show_viewport       = 0;
 
    /* Repeat. */
    conf.repeat_delay = 500;
@@ -147,13 +149,14 @@ void conf_setDefaults( void )
    conf.font_size_small   = FONT_SIZE_SMALL_DEFAULT;
 
    /* Misc. */
-   conf.redirect_file            = 1;
-   conf.nosave                   = 0;
-   conf.devmode                  = 0;
-   conf.devautosave              = 0;
-   conf.lua_enet                 = 0;
-   conf.lua_repl                 = 0;
-   conf.lastversion              = strdup( "" );
+   conf.redirect_file = 1;
+   conf.nosave        = 0;
+   conf.devmode       = 0;
+   conf.devautosave   = 0;
+   conf.lua_enet      = 0;
+   conf.lua_repl      = 0;
+   free( conf.lastversion );
+   conf.lastversion              = NULL;
    conf.translation_warning_seen = 0;
    memset( &conf.last_played, 0, sizeof( time_t ) );
 
@@ -214,20 +217,22 @@ void conf_setAudioDefaults( void )
  */
 void conf_setVideoDefaults( void )
 {
-   int             w, h, f;
-   SDL_DisplayMode resolution;
+   int w, h, f;
+
+   conf.num_backups = NUM_BACKUPS_DEFAULT;
 
    /* More complex resolution handling. */
-   f = 0;
-   if ( SDL_GetCurrentDisplayMode( 0, &resolution ) == 0 ) {
+   f                                 = 0;
+   const SDL_DisplayMode *resolution = SDL_GetCurrentDisplayMode( 0 );
+   if ( resolution != NULL ) {
       /* Try higher resolution. */
       w = RESOLUTION_W_DEFAULT;
       h = RESOLUTION_H_DEFAULT;
 
       /* Fullscreen and fit everything onscreen. */
-      if ( ( resolution.w <= w ) || ( resolution.h <= h ) ) {
-         w = resolution.w;
-         h = resolution.h;
+      if ( ( resolution->w <= w ) || ( resolution->h <= h ) ) {
+         w = resolution->w;
+         h = resolution->h;
          f = FULLSCREEN_DEFAULT;
       }
    } else {
@@ -240,11 +245,9 @@ void conf_setVideoDefaults( void )
    conf.vsync = VSYNC_DEFAULT;
 
    /* Window. */
-   conf.fullscreen = f;
-   conf.width      = w;
-   conf.height     = h;
-   conf.explicit_dim =
-      0; /* No need for a define, this is only for first-run. */
+   conf.fullscreen          = f;
+   conf.width               = w;
+   conf.height              = h;
    conf.scalefactor         = SCALE_FACTOR_DEFAULT;
    conf.nebu_scale          = NEBULA_SCALE_FACTOR_DEFAULT;
    conf.minimize            = MINIMIZE_DEFAULT;
@@ -281,7 +284,7 @@ void conf_cleanup( void )
 }
 
 /*
- * @brief Parses the local conf that dictates where user data goes.
+ * @brief Parses the local configuration that dictates where user data goes.
  */
 void conf_loadConfigPath( void )
 {
@@ -290,11 +293,15 @@ void conf_loadConfigPath( void )
    if ( !nfile_fileExists( file ) )
       return;
 
-   nlua_env lEnv = nlua_newEnv( file );
-   if ( nlua_dofileenv( lEnv, file ) == 0 )
-      conf_loadString( lEnv, "datapath", conf.datapath );
+   lua_State *L = luaL_newstate();
 
-   nlua_freeEnv( lEnv );
+   if ( luaL_dofile( L, file ) != 0 ) {
+      lua_close( L );
+      return;
+   }
+   conf_loadString( L, "datapath", conf.datapath );
+
+   lua_close( L );
 }
 
 /*
@@ -302,11 +309,12 @@ void conf_loadConfigPath( void )
  */
 int conf_loadConfig( const char *file )
 {
-   int         t, cb;
+   int         t;
    SDL_Keycode key;
    int         type;
    int         w, h;
    SDL_Keymod  m;
+   lua_State  *L;
 
    /* Check to see if file exists. */
    if ( !nfile_fileExists( file ) ) {
@@ -315,264 +323,255 @@ int conf_loadConfig( const char *file )
    }
 
    /* Load the configuration. */
-   nlua_env lEnv = nlua_newEnv( file );
-   if ( nlua_dofileenv( lEnv, file ) == 0 ) {
-
-      /* ndata. */
-      conf_loadString( lEnv, "data", conf.ndata );
-
-      /* Language. */
-      conf_loadString( lEnv, "language", conf.language );
-
-      /* OpenGL. */
-      conf_loadInt( lEnv, "fsaa", conf.fsaa );
-      conf_loadBool( lEnv, "vsync", conf.vsync );
-
-      /* Window. */
-      w = h = 0;
-      conf_loadInt( lEnv, "width", w );
-      conf_loadInt( lEnv, "height", h );
-      if ( w != 0 ) {
-         conf.explicit_dim = 1;
-         conf.width        = w;
-      }
-      if ( h != 0 ) {
-         conf.explicit_dim = 1;
-         conf.height       = h;
-      }
-      conf_loadFloat( lEnv, "scalefactor", conf.scalefactor );
-      conf_loadFloat( lEnv, "nebu_scale", conf.nebu_scale );
-      conf_loadBool( lEnv, "fullscreen", conf.fullscreen );
-      conf_loadBool( lEnv, "modesetting", conf.modesetting );
-      conf_loadBool( lEnv, "notresizable", conf.notresizable );
-      conf_loadBool( lEnv, "borderless", conf.borderless );
-      conf_loadBool( lEnv, "minimize", conf.minimize );
-      cb = 0;
-      conf_loadBool( lEnv, "colourblind",
-                     cb ); /* TODO remove in 0.13.0 or so. */
-      if ( cb ) {
-         /* Old colourblind used Rod Monochromancy, so we'll restore that in
-          * this case. TODO Remove in 0.13.0 or so. */
-         conf.colourblind_type = 3;
-         conf.colourblind_sim  = 1.; /* Turn on at max. */
-      }
-      conf_loadFloat( lEnv, "colourblind_sim", conf.colourblind_sim );
-      conf_loadFloat( lEnv, "colourblind_correct", conf.colourblind_correct );
-      conf_loadInt( lEnv, "colourblind_type", conf.colourblind_type );
-      conf_loadFloat( lEnv, "game_speed", conf.game_speed );
-      conf_loadBool( lEnv, "healthbars", conf.healthbars );
-      conf_loadFloat( lEnv, "bg_brightness", conf.bg_brightness );
-      conf_loadBool( lEnv, "puzzle_skip", conf.puzzle_skip );
-      /* TODO leave only nebu_nonuniformity for 0.13.0 */
-      conf_loadFloat( lEnv, "nebu_uniformity", conf.nebu_nonuniformity );
-      conf_loadFloat( lEnv, "nebu_nonuniformity", conf.nebu_nonuniformity );
-      /* end todo */
-      conf_loadFloat( lEnv, "nebu_saturation", conf.nebu_saturation );
-      conf_loadFloat( lEnv, "jump_brightness", conf.jump_brightness );
-      conf_loadFloat( lEnv, "gamma_correction", conf.gamma_correction );
-      conf_loadBool( lEnv, "low_memory", conf.low_memory );
-      conf_loadInt( lEnv, "max_3d_tex_size", conf.max_3d_tex_size );
-
-      /* FPS */
-      conf_loadBool( lEnv, "showfps", conf.fps_show );
-      conf_loadInt( lEnv, "maxfps", conf.fps_max );
-
-      /*  Pause */
-      conf_loadBool( lEnv, "showpause", conf.pause_show );
-
-      /* Sound. */
-      conf_loadBool( lEnv, "al_efx", conf.al_efx );
-      conf_loadBool( lEnv, "nosound", conf.nosound );
-      conf_loadFloat( lEnv, "sound", conf.sound );
-      conf_loadFloat( lEnv, "music", conf.music );
-      conf_loadFloat( lEnv, "engine_vol", conf.engine_vol );
-
-      /* Joystick. */
-      nlua_getenv( naevL, lEnv, "joystick" );
-      if ( lua_isnumber( naevL, -1 ) )
-         conf.joystick_ind = (int)lua_tonumber( naevL, -1 );
-      else if ( lua_isstring( naevL, -1 ) )
-         conf.joystick_nam = strdup( lua_tostring( naevL, -1 ) );
-      lua_pop( naevL, 1 );
-
-      /* GUI. */
-      conf_loadInt( lEnv, "mesg_visible", conf.mesg_visible );
-      if ( conf.mesg_visible <= 0 )
-         conf.mesg_visible = 5;
-      conf_loadFloat( lEnv, "map_overlay_opacity", conf.map_overlay_opacity );
-      conf.map_overlay_opacity = CLAMP( 0, 1, conf.map_overlay_opacity );
-      conf_loadBool( lEnv, "big_icons", conf.big_icons );
-      conf_loadBool( lEnv, "always_radar", conf.always_radar );
-
-      /* Key repeat. */
-      conf_loadInt( lEnv, "repeat_delay", conf.repeat_delay );
-      conf_loadInt( lEnv, "repeat_freq", conf.repeat_freq );
-
-      /* Zoom. */
-      conf_loadBool( lEnv, "zoom_manual", conf.zoom_manual );
-      conf_loadFloat( lEnv, "zoom_far", conf.zoom_far );
-      conf_loadFloat( lEnv, "zoom_near", conf.zoom_near );
-      conf_loadFloat( lEnv, "zoom_speed", conf.zoom_speed );
-
-      /* Font size. */
-      conf_loadInt( lEnv, "font_size_console", conf.font_size_console );
-      conf_loadInt( lEnv, "font_size_intro", conf.font_size_intro );
-      conf_loadInt( lEnv, "font_size_def", conf.font_size_def );
-      conf_loadInt( lEnv, "font_size_small", conf.font_size_small );
-
-      /* Misc. */
-      conf_loadString( lEnv, "difficulty", conf.difficulty );
-      conf_loadFloat( lEnv, "compression_velocity", conf.compression_velocity );
-      conf_loadFloat( lEnv, "compression_mult", conf.compression_mult );
-      conf_loadBool( lEnv, "redirect_file", conf.redirect_file );
-      conf_loadInt( lEnv, "doubletap_sensitivity", conf.doubletap_sens );
-      conf_loadFloat( lEnv, "mouse_hide", conf.mouse_hide );
-      conf_loadBool( lEnv, "mouse_fly", conf.mouse_fly );
-      conf_loadInt( lEnv, "mouse_accel", conf.mouse_accel );
-      conf_loadFloat( lEnv, "mouse_doubleclick", conf.mouse_doubleclick );
-      conf_loadFloat( lEnv, "autonav_reset_dist", conf.autonav_reset_dist );
-      conf_loadFloat( lEnv, "autonav_reset_shield", conf.autonav_reset_shield );
-      conf_loadBool( lEnv, "devmode", conf.devmode );
-      conf_loadBool( lEnv, "devautosave", conf.devautosave );
-      conf_loadBool( lEnv, "lua_enet", conf.lua_enet );
-      conf_loadBool( lEnv, "lua_repl", conf.lua_repl );
-      conf_loadBool( lEnv, "conf_nosave", conf.nosave );
-      conf_loadString( lEnv, "lastversion", conf.lastversion );
-      conf_loadBool( lEnv, "translation_warning_seen",
-                     conf.translation_warning_seen );
-      conf_loadTime( lEnv, "last_played", conf.last_played );
-
-      /* Debugging. */
-      conf_loadBool( lEnv, "fpu_except", conf.fpu_except );
-
-      /* Editor. */
-      conf_loadString( lEnv, "dev_data_dir", conf.dev_data_dir );
-
-      /*
-       * Keybindings.
-       */
-      for ( int i = 0; i <= KST_PASTE; i++ ) {
-         nlua_getenv( naevL, lEnv, input_getKeybindBrief( i ) );
-
-         /* Use 'none' to differentiate between not instantiated and disabled
-          * bindings. */
-         if ( lua_isstring( naevL, -1 ) ) {
-            const char *str = lua_tostring( naevL, -1 );
-            if ( strcmp( str, "none" ) == 0 ) {
-               input_setKeybind( i, KEYBIND_NULL, SDLK_UNKNOWN, NMOD_NONE );
-            }
-         } else if ( lua_istable( naevL, -1 ) ) { /* it's a table */
-            const char *str, *mod;
-            /* gets the event type */
-            lua_getfield( naevL, -1, "type" );
-            if ( lua_isstring( naevL, -1 ) )
-               str = lua_tostring( naevL, -1 );
-            else if ( lua_isnil( naevL, -1 ) ) {
-               WARN( _( "Found keybind with no type field!" ) );
-               str = "null";
-            } else {
-               WARN( _( "Found keybind with invalid type field!" ) );
-               str = "null";
-            }
-            lua_pop( naevL, 1 );
-
-            /* gets the key */
-            lua_getfield( naevL, -1, "key" );
-            t = lua_type( naevL, -1 );
-            if ( t == LUA_TNUMBER )
-               key = (int)lua_tonumber( naevL, -1 );
-            else if ( t == LUA_TSTRING )
-               key = input_keyConv( lua_tostring( naevL, -1 ) );
-            else if ( t == LUA_TNIL ) {
-               WARN( _( "Found keybind with no key field!" ) );
-               key = SDLK_UNKNOWN;
-            } else {
-               WARN( _( "Found keybind with invalid key field!" ) );
-               key = SDLK_UNKNOWN;
-            }
-            lua_pop( naevL, 1 );
-
-            /* Get the modifier. */
-            lua_getfield( naevL, -1, "mod" );
-            if ( lua_isstring( naevL, -1 ) )
-               mod = lua_tostring( naevL, -1 );
-            else
-               mod = NULL;
-            lua_pop( naevL, 1 );
-
-            if ( str != NULL ) { /* keybind is valid */
-               /* get type */
-               if ( strcmp( str, "null" ) == 0 )
-                  type = KEYBIND_NULL;
-               else if ( strcmp( str, "keyboard" ) == 0 )
-                  type = KEYBIND_KEYBOARD;
-               else if ( strcmp( str, "jaxispos" ) == 0 )
-                  type = KEYBIND_JAXISPOS;
-               else if ( strcmp( str, "jaxisneg" ) == 0 )
-                  type = KEYBIND_JAXISNEG;
-               else if ( strcmp( str, "jbutton" ) == 0 )
-                  type = KEYBIND_JBUTTON;
-               else if ( strcmp( str, "jhat_up" ) == 0 )
-                  type = KEYBIND_JHAT_UP;
-               else if ( strcmp( str, "jhat_down" ) == 0 )
-                  type = KEYBIND_JHAT_DOWN;
-               else if ( strcmp( str, "jhat_left" ) == 0 )
-                  type = KEYBIND_JHAT_LEFT;
-               else if ( strcmp( str, "jhat_right" ) == 0 )
-                  type = KEYBIND_JHAT_RIGHT;
-               else {
-                  WARN( _( "Unknown keybinding of type %s" ), str );
-                  continue;
-               }
-
-               /* Check to see if it is valid. */
-               if ( ( key == SDLK_UNKNOWN ) && ( type == KEYBIND_KEYBOARD ) ) {
-                  WARN( _( "Keybind for '%s' is invalid" ),
-                        input_getKeybindName( i ) );
-                  continue;
-               }
-
-               /* Set modifier, probably should be able to handle two at a time.
-                */
-               if ( mod != NULL ) {
-                  if ( strcmp( mod, "ctrl" ) == 0 )
-                     m = NMOD_CTRL;
-                  else if ( strcmp( mod, "shift" ) == 0 )
-                     m = NMOD_SHIFT;
-                  else if ( strcmp( mod, "alt" ) == 0 )
-                     m = NMOD_ALT;
-                  else if ( strcmp( mod, "meta" ) == 0 )
-                     m = NMOD_META;
-                  else if ( strcmp( mod, "any" ) == 0 )
-                     m = NMOD_ANY;
-                  else if ( strcmp( mod, "none" ) == 0 )
-                     m = NMOD_NONE;
-                  else {
-                     WARN( _( "Unknown keybinding mod of type %s" ), mod );
-                     m = NMOD_NONE;
-                  }
-               } else
-                  m = NMOD_NONE;
-
-               /* set the keybind */
-               input_setKeybind( i, type, key, m );
-            } else
-               WARN( _( "Malformed keybind for '%s' in '%s'." ),
-                     input_getKeybindName( i ), file );
-         }
-         /* clean up after table stuff */
-         lua_pop( naevL, 1 );
-      }
-      lua_pop( naevL, 1 );
-   } else { /* failed to load the config file */
+   L = luaL_newstate();
+   // TODO sandbox
+   luaL_openlibs( L );
+   if ( luaL_dofile( L, file ) != 0 ) {
       WARN( _( "Config file '%s' has invalid syntax:" ), file );
-      WARN( "   %s", lua_tostring( naevL, -1 ) );
-      nlua_freeEnv( lEnv );
-      return 1;
+      WARN( "   %s", lua_tostring( L, -1 ) );
+      lua_close( L );
+      return -1;
    }
 
+   /* ndata. */
+   conf_loadString( L, "data", conf.ndata );
+
+   /* Language. */
+   conf_loadString( L, "language", conf.language );
+
+   /* OpenGL. */
+   conf_loadInt( L, "fsaa", conf.fsaa );
+   conf_loadBool( L, "vsync", conf.vsync );
+
+   /* Window. */
+   w = h = 0;
+   conf_loadInt( L, "width", w );
+   conf_loadInt( L, "height", h );
+   if ( w != 0 ) {
+      conf.width = w;
+   }
+   if ( h != 0 ) {
+      conf.height = h;
+   }
+   conf_loadFloat( L, "scalefactor", conf.scalefactor );
+   conf_loadFloat( L, "nebu_scale", conf.nebu_scale );
+   conf_loadBool( L, "fullscreen", conf.fullscreen );
+   conf_loadBool( L, "notresizable", conf.notresizable );
+   conf_loadBool( L, "minimize", conf.minimize );
+   conf_loadFloat( L, "colourblind_sim", conf.colourblind_sim );
+   conf_loadFloat( L, "colourblind_correct", conf.colourblind_correct );
+   conf_loadInt( L, "colourblind_type", conf.colourblind_type );
+   conf_loadFloat( L, "game_speed", conf.game_speed );
+   conf_loadBool( L, "healthbars", conf.healthbars );
+   conf_loadFloat( L, "bg_brightness", conf.bg_brightness );
+   conf_loadBool( L, "puzzle_skip", conf.puzzle_skip );
+   conf_loadFloat( L, "nebu_nonuniformity", conf.nebu_nonuniformity );
+   /* end todo */
+   conf_loadFloat( L, "nebu_saturation", conf.nebu_saturation );
+   conf_loadFloat( L, "jump_brightness", conf.jump_brightness );
+   conf_loadFloat( L, "gamma_correction", conf.gamma_correction );
+   conf_loadBool( L, "low_memory", conf.low_memory );
+   conf_loadInt( L, "max_3d_tex_size", conf.max_3d_tex_size );
+
+   /* FPS */
+   conf_loadBool( L, "showfps", conf.fps_show );
+   conf_loadInt( L, "maxfps", conf.fps_max );
+
+   /*  Pause */
+   conf_loadBool( L, "showpause", conf.pause_show );
+
+   /* Sound. */
+   conf_loadBool( L, "al_efx", conf.al_efx );
+   conf_loadBool( L, "nosound", conf.nosound );
+   conf_loadFloat( L, "sound", conf.sound );
+   conf_loadFloat( L, "music", conf.music );
+   conf_loadFloat( L, "engine_vol", conf.engine_vol );
+
+   /* Joystick. */
+   lua_getglobal( L, "joystick" );
+   if ( lua_isnumber( L, -1 ) )
+      conf.joystick_ind = (int)lua_tonumber( L, -1 );
+   else if ( lua_isstring( L, -1 ) )
+      conf.joystick_nam = strdup( lua_tostring( L, -1 ) );
+   lua_pop( L, 1 );
+
+   /* GUI. */
+   conf_loadInt( L, "mesg_visible", conf.mesg_visible );
+   if ( conf.mesg_visible <= 0 )
+      conf.mesg_visible = 5;
+   conf_loadFloat( L, "map_overlay_opacity", conf.map_overlay_opacity );
+   conf.map_overlay_opacity = CLAMP( 0, 1, conf.map_overlay_opacity );
+   conf_loadBool( L, "big_icons", conf.big_icons );
+   conf_loadBool( L, "always_radar", conf.always_radar );
+   conf_loadBool( L, "show_viewport", conf.show_viewport );
+
+   /* Key repeat. */
+   conf_loadInt( L, "repeat_delay", conf.repeat_delay );
+   conf_loadInt( L, "repeat_freq", conf.repeat_freq );
+
+   /* Zoom. */
+   conf_loadBool( L, "zoom_manual", conf.zoom_manual );
+   conf_loadFloat( L, "zoom_far", conf.zoom_far );
+   conf_loadFloat( L, "zoom_near", conf.zoom_near );
+   conf_loadFloat( L, "zoom_speed", conf.zoom_speed );
+
+   /* Font size. */
+   conf_loadInt( L, "font_size_console", conf.font_size_console );
+   conf_loadInt( L, "font_size_intro", conf.font_size_intro );
+   conf_loadInt( L, "font_size_def", conf.font_size_def );
+   conf_loadInt( L, "font_size_small", conf.font_size_small );
+
+   /* Misc. */
+   conf_loadString( L, "difficulty", conf.difficulty );
+   conf_loadFloat( L, "compression_velocity", conf.compression_velocity );
+   conf_loadFloat( L, "compression_mult", conf.compression_mult );
+   conf_loadBool( L, "redirect_file", conf.redirect_file );
+   conf_loadInt( L, "doubletap_sensitivity", conf.doubletap_sens );
+   conf_loadFloat( L, "mouse_hide", conf.mouse_hide );
+   conf_loadBool( L, "mouse_fly", conf.mouse_fly );
+   conf_loadInt( L, "mouse_accel", conf.mouse_accel );
+   conf_loadFloat( L, "mouse_doubleclick", conf.mouse_doubleclick );
+   conf_loadFloat( L, "autonav_reset_dist", conf.autonav_reset_dist );
+   conf_loadFloat( L, "autonav_reset_shield", conf.autonav_reset_shield );
+   conf_loadBool( L, "devmode", conf.devmode );
+   conf_loadBool( L, "devautosave", conf.devautosave );
+   conf_loadBool( L, "lua_enet", conf.lua_enet );
+   conf_loadBool( L, "lua_repl", conf.lua_repl );
+   conf_loadBool( L, "conf_nosave", conf.nosave );
+   conf_loadString( L, "lastversion", conf.lastversion );
+   conf_loadBool( L, "translation_warning_seen",
+                  conf.translation_warning_seen );
+   conf_loadTime( L, "last_played", conf.last_played );
+
+   /* Debugging. */
+   conf_loadBool( L, "fpu_except", conf.fpu_except );
+
+   /* Editor. */
+   conf_loadString( L, "dev_data_dir", conf.dev_data_dir );
+
+   /*
+    * Keybindings.
+    */
+   for ( int i = 0; i <= KST_PASTE; i++ ) {
+      lua_getglobal( L, input_getKeybindBrief( i ) );
+
+      /* Use 'none' to differentiate between not instantiated and disabled
+       * bindings. */
+      if ( lua_isstring( L, -1 ) ) {
+         const char *str = lua_tostring( L, -1 );
+         if ( strcmp( str, "none" ) == 0 ) {
+            input_setKeybind( i, KEYBIND_NULL, SDLK_UNKNOWN, NMOD_NONE );
+         }
+      } else if ( lua_istable( L, -1 ) ) { /* it's a table */
+         const char *str, *mod;
+         /* gets the event type */
+         lua_getfield( L, -1, "type" );
+         if ( lua_isstring( L, -1 ) )
+            str = lua_tostring( L, -1 );
+         else if ( lua_isnil( L, -1 ) ) {
+            WARN( _( "Found keybind with no type field!" ) );
+            str = "null";
+         } else {
+            WARN( _( "Found keybind with invalid type field!" ) );
+            str = "null";
+         }
+         lua_pop( L, 1 );
+
+         /* gets the key */
+         lua_getfield( L, -1, "key" );
+         t = lua_type( L, -1 );
+         if ( t == LUA_TNUMBER )
+            key = (int)lua_tonumber( L, -1 );
+         else if ( t == LUA_TSTRING ) {
+            const char *name = lua_tostring( L, -1 );
+            key              = input_keyFromStr( name );
+            if ( key == SDLK_UNKNOWN )
+               WARN( _( "Keyname '%s' doesn't match any key." ), name );
+         } else if ( t == LUA_TNIL ) {
+            WARN( _( "Found keybind with no key field!" ) );
+            key = SDLK_UNKNOWN;
+         } else {
+            WARN( _( "Found keybind with invalid key field!" ) );
+            key = SDLK_UNKNOWN;
+         }
+         lua_pop( L, 1 );
+
+         /* Get the modifier. */
+         lua_getfield( L, -1, "mod" );
+         if ( lua_isstring( L, -1 ) )
+            mod = lua_tostring( L, -1 );
+         else
+            mod = NULL;
+         lua_pop( L, 1 );
+
+         if ( str != NULL ) { /* keybind is valid */
+            /* get type */
+            if ( strcmp( str, "null" ) == 0 )
+               type = KEYBIND_NULL;
+            else if ( strcmp( str, "keyboard" ) == 0 )
+               type = KEYBIND_KEYBOARD;
+            else if ( strcmp( str, "jaxispos" ) == 0 )
+               type = KEYBIND_JAXISPOS;
+            else if ( strcmp( str, "jaxisneg" ) == 0 )
+               type = KEYBIND_JAXISNEG;
+            else if ( strcmp( str, "jbutton" ) == 0 )
+               type = KEYBIND_JBUTTON;
+            else if ( strcmp( str, "jhat_up" ) == 0 )
+               type = KEYBIND_JHAT_UP;
+            else if ( strcmp( str, "jhat_down" ) == 0 )
+               type = KEYBIND_JHAT_DOWN;
+            else if ( strcmp( str, "jhat_left" ) == 0 )
+               type = KEYBIND_JHAT_LEFT;
+            else if ( strcmp( str, "jhat_right" ) == 0 )
+               type = KEYBIND_JHAT_RIGHT;
+            else {
+               WARN( _( "Unknown keybinding of type %s" ), str );
+               continue;
+            }
+
+            /* Check to see if it is valid. */
+            if ( ( key == SDLK_UNKNOWN ) && ( type == KEYBIND_KEYBOARD ) ) {
+               WARN( _( "Keybind for '%s' is invalid" ),
+                     input_getKeybindName( i ) );
+               continue;
+            }
+
+            /* Set modifier, probably should be able to handle two at a time.
+             */
+            if ( mod != NULL ) {
+               if ( strcmp( mod, "ctrl" ) == 0 )
+                  m = NMOD_CTRL;
+               else if ( strcmp( mod, "shift" ) == 0 )
+                  m = NMOD_SHIFT;
+               else if ( strcmp( mod, "alt" ) == 0 )
+                  m = NMOD_ALT;
+               else if ( strcmp( mod, "meta" ) == 0 )
+                  m = NMOD_META;
+               else if ( strcmp( mod, "any" ) == 0 )
+                  m = NMOD_ANY;
+               else if ( strcmp( mod, "none" ) == 0 )
+                  m = NMOD_NONE;
+               else {
+                  WARN( _( "Unknown keybinding mod of type %s" ), mod );
+                  m = NMOD_NONE;
+               }
+            } else
+               m = NMOD_NONE;
+
+            /* set the keybind */
+            input_setKeybind( i, type, key, m );
+         } else
+            WARN( _( "Malformed keybind for '%s' in '%s'." ),
+                  input_getKeybindName( i ), file );
+      }
+      /* clean up after table stuff */
+      lua_pop( L, 1 );
+   }
+   lua_pop( L, 1 );
+
    conf.loaded = 1;
-   nlua_freeEnv( lEnv );
+
+   lua_close( L );
    return 0;
 }
 
@@ -596,8 +595,10 @@ int conf_parseCLI( int argc, char **argv )
       { "svol", required_argument, 0, 's' },
       { "scale", required_argument, 0, 'X' },
       { "devmode", no_argument, 0, 'D' },
+      { "pluginmanager", no_argument, 0, 'p' },
       { "help", no_argument, 0, 'h' },
       { "version", no_argument, 0, 'v' },
+      { "exitmainmenu", no_argument, 0, '\e' },
       { NULL, 0, 0, 0 } };
    int option_index = 1;
    int c            = 0;
@@ -607,7 +608,7 @@ int conf_parseCLI( int argc, char **argv )
     * option.
     */
    optind = 0;
-   while ( ( c = getopt_long( argc, argv, "fF:Vd:j:J:W:H:MSm:s:X:Nhv",
+   while ( ( c = getopt_long( argc, argv, "fF:Vd:j:J:W:H:MSm:s:X:Nphv",
                               long_options, &option_index ) ) != -1 ) {
       switch ( c ) {
       case 'd':
@@ -629,12 +630,10 @@ int conf_parseCLI( int argc, char **argv )
          conf.joystick_nam = strdup( optarg );
          break;
       case 'W':
-         conf.width        = atoi( optarg );
-         conf.explicit_dim = 1;
+         conf.width = atoi( optarg );
          break;
       case 'H':
-         conf.height       = atoi( optarg );
-         conf.explicit_dim = 1;
+         conf.height = atoi( optarg );
          break;
       case 'M':
          conf.nosound = 1;
@@ -659,6 +658,13 @@ int conf_parseCLI( int argc, char **argv )
          conf.devmode = 1;
          LOG( _( "Enabling developer mode." ) );
          break;
+      case 'p':
+         /* Handled on the Rust side to spin up the GUI; parsed here to avoid
+          * getopt noise on platforms like macOS. */
+         break;
+      case '\e':
+         conf.exit_main_menu = 1;
+         break;
 
       case 'v':
          /* by now it has already displayed the version */
@@ -677,9 +683,9 @@ int conf_parseCLI( int argc, char **argv )
  * source code
  *
  *    @param str The destination buffer
- *    @param size The maximum amount of space in str to use
+ *    @param size The maximum amount of space in string to use
  *    @param text The string to quote and escape
- *    @return The number of characters actually written to str
+ *    @return The number of characters actually written to string
  */
 static size_t quoteLuaString( char *str, size_t size, const char *text )
 {
@@ -774,6 +780,12 @@ static size_t quoteLuaString( char *str, size_t size, const char *text )
 
 #define conf_saveComment( t )                                                  \
    pos += scnprintf( &buf[pos], sizeof( buf ) - pos, "-- %s\n", t );
+#define conf_saveCommentVar( str, ... )                                        \
+   do {                                                                        \
+      char _BUF[STRMAX_SHORT];                                                 \
+      scnprintf( _BUF, sizeof( _BUF ), str, ##__VA_ARGS__ );                   \
+      pos += scnprintf( &buf[pos], sizeof( buf ) - pos, "-- %s\n", _BUF );     \
+   } while ( 0 )
 
 #define conf_saveEmptyLine()                                                   \
    if ( sizeof( buf ) != pos )                                                 \
@@ -831,14 +843,15 @@ int conf_saveConfig( const char *file )
       /* See if we can find the generated section and preserve
        * whatever the user wrote before it */
       const char *tmp =
-         strnstr( old, "-- " GENERATED_START_COMMENT "\n", oldsize );
+         SDL_strnstr( old, "-- " GENERATED_START_COMMENT "\n", oldsize );
       if ( tmp != NULL ) {
          /* Copy over the user content */
          pos = MIN( sizeof( buf ), (size_t)( tmp - old ) );
          memcpy( buf, old, pos );
 
          /* See if we can find the end of the section */
-         tmp = strnstr( tmp, "-- " GENERATED_END_COMMENT "\n", oldsize - pos );
+         tmp =
+            SDL_strnstr( tmp, "-- " GENERATED_END_COMMENT "\n", oldsize - pos );
          if ( tmp != NULL ) {
             /* Everything after this should also be preserved */
             oldfooter = tmp + strlen( "-- " GENERATED_END_COMMENT "\n" );
@@ -874,6 +887,11 @@ int conf_saveConfig( const char *file )
    conf_saveString( "data", conf.ndata );
    conf_saveEmptyLine();
 
+   /* OpenGL. */
+   conf_saveComment( _( "Number of save game backups" ) );
+   conf_saveInt( "num_backups", conf.num_backups );
+   conf_saveEmptyLine();
+
    /* Language. */
    conf_saveComment(
       _( "Language to use. Set to the two character identifier to the language "
@@ -908,13 +926,8 @@ int conf_saveConfig( const char *file )
    conf_saveComment( _( "The window size or screen resolution" ) );
    conf_saveComment(
       _( "Set both of these to 0 to make Naev try the desktop resolution" ) );
-   if ( conf.explicit_dim ) {
-      conf_saveInt( "width", conf.width );
-      conf_saveInt( "height", conf.height );
-   } else {
-      conf_saveInt( "width", 0 );
-      conf_saveInt( "height", 0 );
-   }
+   conf_saveInt( "width", conf.width );
+   conf_saveInt( "height", conf.height );
    conf_saveEmptyLine();
 
    conf_saveComment( _( "Factor used to divide the above resolution with" ) );
@@ -933,20 +946,10 @@ int conf_saveConfig( const char *file )
    conf_saveBool( "fullscreen", conf.fullscreen );
    conf_saveEmptyLine();
 
-   conf_saveComment( _( "Use video modesetting when fullscreen is enabled" ) );
-   conf_saveBool( "modesetting", conf.modesetting );
-   conf_saveEmptyLine();
-
    conf_saveComment( _( "Disable allowing resizing the window." ) );
    conf_saveBool( "notresizable", conf.notresizable );
    conf_saveEmptyLine();
 
-   conf_saveComment(
-      _( "Disable window decorations. Use with care and know the keyboard "
-         "controls to quit and toggle fullscreen." ) );
-   conf_saveBool( "borderless", conf.borderless );
-   conf_saveEmptyLine();
-
    conf_saveComment( _( "Minimize the game on focus loss." ) );
    conf_saveBool( "minimize", conf.minimize );
    conf_saveEmptyLine();
@@ -979,9 +982,10 @@ int conf_saveConfig( const char *file )
    conf_saveBool( "healthbars", conf.healthbars );
    conf_saveEmptyLine();
 
-   conf_saveComment(
-      _( "Background brightness. 1 is normal brightness while setting it to 0 "
-         "would make the backgrounds pitch black." ) );
+   conf_saveCommentVar(
+      _( "Background brightness. 1 is full brightness while setting it to 0 "
+         "would make the backgrounds pitch black. Defaults to %.1f." ),
+      BG_BRIGHTNESS_DEFAULT );
    conf_saveFloat( "bg_brightness", conf.bg_brightness );
    conf_saveEmptyLine();
 
@@ -1051,7 +1055,7 @@ int conf_saveConfig( const char *file )
    conf_saveComment(
       _( "Volume of sound effects and music, between 0.0 and 1.0" ) );
    conf_saveFloat( "sound",
-                   ( sound_disabled ) ? conf.sound : sound_getVolume() );
+                   ( sound_disabled() ) ? conf.sound : sound_getVolume() );
    conf_saveFloat( "music",
                    ( music_disabled ) ? conf.music : music_getVolume() );
    conf_saveComment(
@@ -1082,6 +1086,8 @@ int conf_saveConfig( const char *file )
    conf_saveComment( _(
       "Always show the radar and don't hide it when the overlay is active." ) );
    conf_saveBool( "always_radar", conf.always_radar );
+   conf_saveComment( _( "Show the viewport in the radar/overlay." ) );
+   conf_saveBool( "show_viewport", conf.show_viewport );
    conf_saveEmptyLine();
 
    /* Key repeat. */
@@ -1291,7 +1297,7 @@ int conf_saveConfig( const char *file )
       /* Determine the textual name for the key, if a keyboard keybind */
       if ( type == KEYBIND_KEYBOARD )
          quoteLuaString( keyname, sizeof( keyname ) - 1,
-                         SDL_GetKeyName( key ) );
+                         input_keyToStr( key ) );
       /* If SDL can't describe the key, store it as an integer */
       if ( type != KEYBIND_KEYBOARD ||
            strcmp( keyname, "\"unknown key\"" ) == 0 )
