@@ -60,22 +60,22 @@ typedef struct ShipWidgetData_ {
    GLuint       tex;
    GLuint       texd;
    double       s;
+   double       dir;
 } ShipWidgetData;
 
 /*
  * equipment stuff
  */
-static CstSlotWidget eq_wgt;                  /**< Equipment widget. */
-static double        equipment_dir      = 0.; /**< Equipment dir. */
-static unsigned int  equipment_lastick  = 0;  /**< Last tick. */
-static unsigned int  equipment_wid      = 0;  /**< Global wid. */
-static int           equipment_creating = 0;  /**< Whether or not creating. */
-static int           ship_mode          = 0;  /**< Ship mode. */
+static CstSlotWidget eq_wgt;                 /**< Equipment widget. */
+static unsigned int  equipment_lastick  = 0; /**< Last tick. */
+static unsigned int  equipment_wid      = 0; /**< Global wid. */
+static int           equipment_creating = 0; /**< Whether or not creating. */
+static int           ship_mode          = 0; /**< Ship mode. */
 static iar_data_t iar_data[OUTFIT_TABS];  /**< Stored image array positions. */
 static Outfit **iar_outfits[OUTFIT_TABS]; /**< Outfits associated with the image
                                              array cells. */
-static nlua_env autoequip_env        = LUA_NOREF; /* Autoequip env. */
-static int      equipment_outfitMode = 0; /**< Outfit mode for filtering. */
+static nlua_env *autoequip_env        = NULL;
+static int       equipment_outfitMode = 0; /**< Outfit mode for filtering. */
 
 /*
  * prototypes
@@ -168,7 +168,7 @@ void equipment_rightClickOutfits( unsigned int wid, const char *str )
       return;
 
    /* Figure out which slot this stuff fits into */
-   switch ( o->slot.type ) {
+   switch ( outfit_slotType( o ) ) {
    case OUTFIT_SLOT_STRUCTURE:
       slots = eq_wgt.selected->p->outfit_structure;
       break;
@@ -192,7 +192,7 @@ void equipment_rightClickOutfits( unsigned int wid, const char *str )
          continue;
 
       /* Must have valid slot size. */
-      if ( o->slot.size == OUTFIT_SLOT_SIZE_NA )
+      if ( outfit_slotSize( o ) == OUTFIT_SLOT_SIZE_NA )
          continue;
 
       minimal = i;
@@ -213,7 +213,7 @@ void equipment_rightClickOutfits( unsigned int wid, const char *str )
    }
 
    /* See if limit is applied and swap with shared limit slot. */
-   if ( o->limit != NULL ) {
+   if ( outfit_limit( o ) != NULL ) {
       minimal = n;
       for ( int i = 0; i < n; i++ ) {
          /* Must fit the slot. */
@@ -221,16 +221,17 @@ void equipment_rightClickOutfits( unsigned int wid, const char *str )
             continue;
 
          /* Must have valid slot size. */
-         if ( o->slot.size == OUTFIT_SLOT_SIZE_NA )
+         if ( outfit_slotSize( o ) == OUTFIT_SLOT_SIZE_NA )
             continue;
 
          /* Must have outfit with limit. */
          if ( ( slots[i].outfit == NULL ) ||
-              ( slots[i].outfit->limit == NULL ) )
+              ( outfit_limit( slots[i].outfit ) == NULL ) )
             continue;
 
          /* Must share a limit to be able to swap. */
-         if ( strcmp( slots[i].outfit->limit, o->limit ) != 0 )
+         if ( strcmp( outfit_limit( slots[i].outfit ), outfit_limit( o ) ) !=
+              0 )
             continue;
 
          minimal = i;
@@ -260,7 +261,7 @@ void equipment_rightClickOutfits( unsigned int wid, const char *str )
          continue;
 
       /* Must have valid slot size. */
-      if ( o->slot.size == OUTFIT_SLOT_SIZE_NA )
+      if ( outfit_slotSize( o ) == OUTFIT_SLOT_SIZE_NA )
          continue;
 
       /* Search for the smallest slot avaliable. */
@@ -349,19 +350,19 @@ void equipment_open( unsigned int wid )
    x = -20;
    y = 20;
    window_addButtonKey( wid, x, y, bw, bh, "btnCloseEquipment", _( "Take Off" ),
-                        land_buttonTakeoff, SDLK_t );
+                        land_buttonTakeoff, SDLK_T );
    x -= ( 15 + bw );
    window_addButtonKey( wid, x, y, bw, bh, "btnSellShip", _( "Sell Ship" ),
-                        equipment_sellShip, SDLK_s );
+                        equipment_sellShip, SDLK_S );
    x -= ( 15 + bw );
    window_addButtonKey( wid, x, y, bw, bh, "btnChangeShip", _( "Swap Ship" ),
-                        equipment_transChangeShip, SDLK_p );
+                        equipment_transChangeShip, SDLK_P );
    x -= ( 15 + bw );
    window_addButtonKey( wid, x, y, bw, bh, "btnUnequipShip", _( "Unequip" ),
-                        equipment_unequipShip, SDLK_u );
+                        equipment_unequipShip, SDLK_U );
    x -= ( 15 + bw );
    window_addButtonKey( wid, x, y, bw, bh, "btnAutoequipShip", _( "Autoequip" ),
-                        equipment_autoequipShip, SDLK_a );
+                        equipment_autoequipShip, SDLK_A );
 
    /* Prepare the outfit array. */
    for ( int i = 0; i < OUTFIT_TABS; i++ ) {
@@ -372,7 +373,6 @@ void equipment_open( unsigned int wid )
 
    /* Safe defaults. */
    equipment_lastick = SDL_GetTicks();
-   equipment_dir     = 0.;
 
    /* Add ammo. */
    equipment_addAmmo();
@@ -408,10 +408,10 @@ void equipment_open( unsigned int wid )
    x = -16;
    y -= 23 + 10;
    window_addButtonKey( wid, x, y, 128 + 8, bh, "btnRenameShip", _( "Rename" ),
-                        equipment_renameShip, SDLK_r );
+                        equipment_renameShip, SDLK_R );
    y -= bh + 10;
    window_addButtonKey( wid, x, y, 128 + 8, bh, "btnShipMode",
-                        _( "Toggle Display" ), equipment_shipMode, SDLK_d );
+                        _( "Toggle Display" ), equipment_shipMode, SDLK_D );
 
    /* Generate lists. */
    equipment_genLists( wid );
@@ -431,7 +431,7 @@ void equipment_open( unsigned int wid )
    window_canFocusWidget( wid, "cstMisc", 0 );
 
    /* Spinning ship. */
-   swd      = malloc( sizeof( ShipWidgetData ) );
+   swd      = calloc( 1, sizeof( ShipWidgetData ) );
    swd->wid = wid;
    swd->s   = ship_maxSize() / gl_screen.scale;
    gl_fboCreate( &swd->fbo, &swd->tex, swd->s, swd->s );
@@ -488,7 +488,7 @@ static void equipment_renderColumn( double x, double y, double w, double h,
       return;
 
    /* Render text. */
-   if ( ( o != NULL ) && ( lst[0].sslot->slot.type == o->slot.type ) )
+   if ( ( o != NULL ) && ( lst[0].sslot->slot.type == outfit_slotType( o ) ) )
       c = &cFontGreen;
    else
       c = &cFontWhite;
@@ -518,7 +518,7 @@ static void equipment_renderColumn( double x, double y, double w, double h,
          else
             dc = &cFontGrey;
       } else
-         dc = outfit_slotSizeColour( &lst[i].sslot->slot );
+         dc = outfit_slotSizeColour( lst[i].sslot->slot.size );
 
       if ( dc == NULL )
          dc = &cGrey60;
@@ -530,14 +530,13 @@ static void equipment_renderColumn( double x, double y, double w, double h,
       else
          bc = *dc;
       bc.a = 0.4;
-      toolkit_drawRect( x, y, w, h, &bc, NULL );
+      toolkit_drawRect( x, y, w, h, &bc );
 
       if ( lst[i].outfit != NULL ) {
-         outfit_gfxStoreLoad( (Outfit *)lst[i].outfit );
          /* Draw bugger. */
-         gl_renderScale( lst[i].outfit->gfx_store, x, y, w, h, NULL );
+         gl_renderScale( outfit_gfxStore( lst[i].outfit ), x, y, w, h, NULL );
       } else if ( ( o != NULL ) &&
-                  ( lst[i].sslot->slot.type == o->slot.type ) ) {
+                  ( lst[i].sslot->slot.type == outfit_slotType( o ) ) ) {
          /* Render a thick frame with a yes/no colour, and geometric cue. */
          int ok = ( !lst[i].sslot->locked &&
                     pilot_canEquip( p, &lst[i], o ) == NULL );
@@ -561,9 +560,9 @@ static void equipment_renderColumn( double x, double y, double w, double h,
 
       /* Draw outline. */
       if ( i == selected )
-         toolkit_drawOutlineThick( x, y, w, h, 5, 7, &cGreen, NULL );
+         toolkit_drawOutlineThick( x, y, w, h, 5, 7, &cGreen );
       if ( rc != NULL )
-         toolkit_drawOutlineThick( x, y, w, h, 1, 3, rc, NULL );
+         toolkit_drawOutlineThick( x, y, w, h, 1, 3, rc );
 
       /* Draw slot iccon if applicable. */
       icon = sp_icon( lst[i].sslot->slot.spid );
@@ -573,7 +572,7 @@ static void equipment_renderColumn( double x, double y, double w, double h,
          double sx = x + w - 10.;
          double sy = y + h - 10.;
 
-         if ( icon->flags & OPENGL_TEX_SDF )
+         if ( tex_isSDF( icon ) )
             gl_renderSDF( icon, sx, sy, sw, sh, &cWhite, 0., 1. );
          else
             gl_renderScaleAspect( icon, sx, sy, sw, sh, NULL );
@@ -708,9 +707,9 @@ static void equipment_renderMisc( double bx, double by, double bw, double bh,
       percent = CLAMP( 0., 1.,
                        1. - (double)player.fleet_used /
                                (double)player.fleet_capacity );
-      toolkit_drawRect( x, y - 2, w * percent, h + 4, &cBlue, NULL );
+      toolkit_drawRect( x, y - 2, w * percent, h + 4, &cBlue );
       toolkit_drawRect( x + w * percent, y - 2, w * ( 1. - percent ), h + 4,
-                        &cBlack, NULL );
+                        &cBlack );
       gl_printMid( &gl_smallFont, w, x, y + h / 2. - gl_smallFont.h / 2.,
                    &cFontWhite, "%d / %d",
                    player.fleet_capacity - player.fleet_used,
@@ -726,9 +725,9 @@ static void equipment_renderMisc( double bx, double by, double bw, double bh,
    percent = ( p->cpu_max > 0 )
                 ? CLAMP( 0., 1., (double)p->cpu / (double)p->cpu_max )
                 : 0.;
-   toolkit_drawRect( x, y - 2, w * percent, h + 4, &cGreen, NULL );
-   toolkit_drawRect( x + w * percent, y - 2, w * ( 1. - percent ), h + 4, &cRed,
-                     NULL );
+   toolkit_drawRect( x, y - 2, w * percent, h + 4, &cGreen );
+   toolkit_drawRect( x + w * percent, y - 2, w * ( 1. - percent ), h + 4,
+                     &cRed );
    gl_printMid( &gl_smallFont, w, x, y + h / 2. - gl_smallFont.h / 2.,
                 &cFontWhite, "%d / %d", p->cpu, p->cpu_max );
 
@@ -744,15 +743,16 @@ static void equipment_renderMisc( double bx, double by, double bw, double bh,
                          ( p->stats.engine_limit - p->solid.mass ) /
                             p->stats.engine_limit )
                 : 0.;
-   toolkit_drawRect( x, y - 2, w * percent, h + 4, &cGreen, NULL );
+   toolkit_drawRect( x, y - 2, w * percent, h + 4, &cGreen );
    toolkit_drawRect( x + w * percent, y - 2, w * ( 1. - percent ), h + 4,
-                     &cOrange, NULL );
+                     &cOrange );
    gl_printMid( &gl_smallFont, w, x, y + h / 2. - gl_smallFont.h / 2.,
                 &cFontWhite, "%.0f / %.0f",
                 p->stats.engine_limit - p->solid.mass, p->stats.engine_limit );
 
    y -= h;
-   if ( p->stats.engine_limit > 0. && p->solid.mass > p->stats.engine_limit ) {
+   if ( p->stats.engine_limit > 0. && p->solid.mass > p->stats.engine_limit &&
+        ( p->speed_base > 0. ) ) {
       gl_printMid( &gl_smallFont, w, x, y, &cFontRed,
                    _( "!! %.0f%% Slower !!" ),
                    ( 1. - p->speed / p->speed_base ) * 100 );
@@ -812,7 +812,8 @@ static void equipment_renderOverlayColumn( double x, double y, double h,
                   c       = &cFontGreen;
                }
             } else if ( ( wgt->outfit != NULL ) &&
-                        ( lst->sslot->slot.type == wgt->outfit->slot.type ) ) {
+                        ( lst->sslot->slot.type ==
+                          outfit_slotType( wgt->outfit ) ) ) {
                top = 1;
                display =
                   pilot_canEquip( wgt->selected->p, &lst[i], wgt->outfit );
@@ -838,7 +839,7 @@ static void equipment_renderOverlayColumn( double x, double y, double h,
             tc.b = 0.;
             tc.a = 0.9;
             toolkit_drawRect( x, y - 5. + yoff, text_width + 60,
-                              gl_smallFont.h + 10, &tc, NULL );
+                              gl_smallFont.h + 10, &tc );
             gl_printMaxRaw( &gl_smallFont, text_width, x + 5, y + yoff, c, -1.,
                             display );
          }
@@ -911,15 +912,17 @@ static void equipment_renderOverlaySlots( double bx, double by, double bw,
    if ( o == NULL ) {
       int pos;
       if ( slot->sslot->slot.spid ) {
-         pos = scnprintf( alt, sizeof( alt ), "#o%s\n",
-                          _( sp_display( slot->sslot->slot.spid ) ) );
+         pos =
+            scnprintf( alt, sizeof( alt ), "#o%s\n",
+                       pgettext_var( "slotproperty",
+                                     sp_display( slot->sslot->slot.spid ) ) );
       } else
          pos = 0;
       pos +=
          scnprintf( &alt[pos], sizeof( alt ) - pos, _( "#%c%s #%c%s #0slot" ),
-                    outfit_slotSizeColourFont( &slot->sslot->slot ),
+                    outfit_slotSizeColourFont( slot->sslot->slot.size ),
                     _( slotSize( slot->sslot->slot.size ) ),
-                    outfit_slotTypeColourFont( &slot->sslot->slot ),
+                    outfit_slotTypeColourFont( slot->sslot->slot.type ),
                     _( slotName( slot->sslot->slot.type ) ) );
       if ( slot->sslot->exclusive && ( pos < (int)sizeof( alt ) ) )
          pos +=
@@ -929,16 +932,18 @@ static void equipment_renderOverlaySlots( double bx, double by, double bw,
                            _( " [locked]" ) );
       if ( slot->sslot->slot.spid )
          scnprintf( &alt[pos], sizeof( alt ) - pos, "\n\n%s",
-                    _( sp_description( slot->sslot->slot.spid ) ) );
+                    pgettext_var( "slotproperty",
+                                  sp_description( slot->sslot->slot.spid ) ) );
       toolkit_drawAltText( bx + wgt->altx, by + wgt->alty, alt );
       return;
    }
 
    /* Get text. */
-   outfit_altText( alt, sizeof( alt ), o, ( p == player.p ) ? p : NULL );
+   outfit_altText( alt, sizeof( alt ), o, p, slot );
 
    /* Display temporary bonuses. */
-   if ( slot->lua_mem != LUA_NOREF ) {
+   if ( ( slot->lua_mem != LUA_NOREF ) &&
+        !outfit_isProp( slot->outfit, OUTFIT_PROP_HIDESTATS ) ) {
       size_t slen = strlen( alt );
       ss_statsListDesc( slot->lua_stats, &alt[slen], sizeof( alt ) - slen, 1 );
    }
@@ -959,13 +964,13 @@ static void equipment_renderOverlaySlots( double bx, double by, double bw,
 static void equipment_renderShip( double bx, double by, double bw, double bh,
                                   void *data )
 {
-   Pilot                *p;
-   int                   s;
-   double                px, py, pw, ph;
-   vec2                  v;
-   GLint                 fbo;
-   const ShipWidgetData *swd = data;
-   const unsigned int    wid = swd->wid;
+   Pilot             *p;
+   int                s;
+   double             px, py, pw, ph;
+   vec2               v;
+   GLint              fbo;
+   ShipWidgetData    *swd = data;
+   const unsigned int wid = swd->wid;
 
    /* Must have selected ship. */
    if ( eq_wgt.selected == NULL )
@@ -977,13 +982,14 @@ static void equipment_renderShip( double bx, double by, double bw, double bh,
       unsigned int tick = SDL_GetTicks();
       double       dt   = (double)( tick - equipment_lastick ) / 1000.;
       equipment_lastick = tick;
-      p->solid.dir += p->turn * dt;
-      if ( p->solid.dir > 2. * M_PI )
-         p->solid.dir = fmod( p->solid.dir, 2. * M_PI );
+      swd->dir += p->turn * dt;
+      if ( swd->dir > 2. * M_PI )
+         swd->dir = fmod( swd->dir, 2. * M_PI );
    } else
       equipment_lastick = SDL_GetTicks();
-   gl_getSpriteFromDir( &p->tsx, &p->tsy, p->ship->sx, p->ship->sy,
-                        p->solid.dir );
+   double tempdir = p->solid.dir;
+   p->solid.dir   = swd->dir;
+   gl_getSpriteFromDir( &p->tsx, &p->tsy, p->ship->sx, p->ship->sy, swd->dir );
 
    s = p->ship->size;
 
@@ -999,17 +1005,28 @@ static void equipment_renderShip( double bx, double by, double bw, double bh,
    py = by + ( bh - ph ) / 2;
 
    /* Render background. */
-   gl_renderRect( px, py, pw, ph, &cBlack );
+   gl_renderRect( bx, by, bw, bh, &cBlack );
 
    /* Use framebuffer to draw, have to use an additional one. */
    s = ceil(
       s / gl_screen.scale ); /* Have to correct for the true rendered size. */
    glGetIntegerv( GL_FRAMEBUFFER_BINDING, &fbo );
-   pilot_renderFramebuffer( p, swd->fbo, gl_screen.nw, gl_screen.nh,
-                            &L_store_const );
+   /* Brute force aproach to clearing the buffer because everything is wrong.
+    * Shouldn't too bad because it's only here that is the issue.
+    * Next three lines shouldn't exist... */
+   glBindFramebuffer( GL_FRAMEBUFFER, swd->fbo );
+   glDisable( GL_SCISSOR_TEST );
+   glClear( GL_COLOR_BUFFER_BIT );
+   ship_renderFramebuffer( p->ship, swd->fbo, gl_screen.nw, gl_screen.nh,
+                           swd->dir, 0., 0., p->r, p->tsx, p->tsy, &cWhite,
+                           &L_store_const );
    glBindFramebuffer( GL_FRAMEBUFFER, fbo );
-   gl_renderTextureRaw( swd->tex, 0, px, py, pw, ph, 0., 0., s / swd->s,
-                        s / swd->s, NULL, 0. );
+   // This is super hacky and we should fix this...
+   mat4 projection = gl_view_matrix;
+   mat4_translate_scale_xy( &projection, px, py, pw, ph );
+   mat4 tex_mat = mat4_identity();
+   mat4_scale_xy( &tex_mat, s / swd->s, s / swd->s );
+   gl_renderTextureRawH( swd->tex, 0, &projection, &tex_mat, NULL );
 
 #ifdef DEBUGGING
    if ( debug_isFlag( DEBUG_MARK_EMITTER ) ) {
@@ -1021,17 +1038,18 @@ static void equipment_renderShip( double bx, double by, double bw, double bh,
 
       if ( use_3d ) {
          H = mat4_identity();
+         /*
          // H.m[2][2] = -1.;
          if ( fabs( p->tilt ) > DOUBLE_TOL ) {
             mat4_rotate( &H, M_PI_2, 0.0, 1.0, 0.0 );
             mat4_rotate( &H, p->tilt, 1.0, 0.0, 0.0 );
-            mat4_rotate( &H, -p->solid.dir, 0.0, 1.0, 0.0 );
-         } else
-            mat4_rotate( &H, -p->solid.dir + M_PI_2, 0.0, 1.0, 0.0 );
+            mat4_rotate( &H, -swd->dir, 0.0, 1.0, 0.0 );
+         } else */
+         mat4_rotate( &H, -swd->dir + M_PI_2, 0.0, 1.0, 0.0 );
          mat4_rotate( &H, -M_PI / 4.0, 1., 0., 0. );
       } else {
-         dircos = cos( equipment_dir );
-         dirsin = sin( equipment_dir );
+         dircos = cos( swd->dir );
+         dirsin = sin( swd->dir );
       }
 
       for ( i = 0; i < array_size( p->ship->trail_emitters ); i++ ) {
@@ -1075,6 +1093,8 @@ static void equipment_renderShip( double bx, double by, double bw, double bh,
       gl_renderShader( px + v.x, py + v.y, 7., 7., 0., &shaders.crosshairs,
                        &cRadar_player, 1 );
    }
+
+   p->solid.dir = tempdir;
 }
 
 static void equipment_freeShipData( unsigned int wid, const char *unused )
@@ -1136,7 +1156,7 @@ static int equipment_mouseColumn( unsigned int wid, const SDL_Event *event,
    if ( ret < 0 )
       return 0;
 
-   if ( event->type == SDL_MOUSEBUTTONDOWN ) {
+   if ( event->type == SDL_EVENT_MOUSE_BUTTON_DOWN ) {
       /* Normal mouse usage. */
       if ( wgt->weapons < 0 ) {
          if ( event->button.button == SDL_BUTTON_LEFT ) {
@@ -1166,7 +1186,7 @@ static int equipment_mouseColumn( unsigned int wid, const SDL_Event *event,
                   /* TODO no freeing. */
                   active   = window_tabWinGetActive( equipment_wid,
                                                      EQUIPMENT_OUTFIT_TAB );
-                  outfits  = array_create( Outfit  *);
+                  outfits  = array_create( Outfit * );
                   noutfits = player_getOutfitsFiltered(
                      (const Outfit ***)&outfits, tabfilters[active],
                      filtertext );
@@ -1263,8 +1283,8 @@ static int equipment_mouseSlots( unsigned int wid, const SDL_Event *event,
    p = wgt->selected->p;
 
    /* Must be left click for now. */
-   if ( ( event->type != SDL_MOUSEBUTTONDOWN ) &&
-        ( event->type != SDL_MOUSEMOTION ) )
+   if ( ( event->type != SDL_EVENT_MOUSE_BUTTON_DOWN ) &&
+        ( event->type != SDL_EVENT_MOUSE_MOTION ) )
       return 0;
 
    /* Is covered by something. */
@@ -1285,7 +1305,7 @@ static int equipment_mouseSlots( unsigned int wid, const SDL_Event *event,
       int ret = equipment_mouseColumn( wid, event, mx, my, y, h,
                                        p->outfit_structure, p, selected, wgt );
       if ( ret )
-         return !!( event->type == SDL_MOUSEBUTTONDOWN );
+         return !!( event->type == SDL_EVENT_MOUSE_BUTTON_DOWN );
    }
    selected += array_size( p->outfit_structure );
    x -= tw;
@@ -1293,7 +1313,7 @@ static int equipment_mouseSlots( unsigned int wid, const SDL_Event *event,
       int ret = equipment_mouseColumn( wid, event, mx, my, y, h,
                                        p->outfit_utility, p, selected, wgt );
       if ( ret )
-         return !!( event->type == SDL_MOUSEBUTTONDOWN );
+         return !!( event->type == SDL_EVENT_MOUSE_BUTTON_DOWN );
    }
    selected += array_size( p->outfit_utility );
    x -= tw;
@@ -1301,7 +1321,7 @@ static int equipment_mouseSlots( unsigned int wid, const SDL_Event *event,
       int ret = equipment_mouseColumn( wid, event, mx, my, y, h,
                                        p->outfit_weapon, p, selected, wgt );
       if ( ret )
-         return !!( event->type == SDL_MOUSEBUTTONDOWN );
+         return !!( event->type == SDL_EVENT_MOUSE_BUTTON_DOWN );
    }
 
    /* Not over anything. */
@@ -1327,6 +1347,10 @@ static int equipment_swapSlot( unsigned int wid, Pilot *p,
       int           ret;
       const Outfit *o = slot->outfit;
 
+      /* Must not be locked. */
+      if ( slot->sslot->locked )
+         return 0;
+
       /* Must be able to remove. */
       if ( pilot_canEquip( eq_wgt.selected->p, slot, NULL ) != NULL )
          return 0;
@@ -1380,12 +1404,8 @@ static int equipment_swapSlot( unsigned int wid, Pilot *p,
          pilot_addOutfitRaw( eq_wgt.selected->p, o, slot );
 
          /* Recalculate stats. */
-         if ( eq_wgt.selected->p->id > 0 )
-            /* TODO there are cases, like outfits that conditionally add stats,
-             * that we would want to actually initialize the Lua. However, the
-             * Lua API requires a pilot to have an ID, so we would need to make
-             * that requirement lax. Maybe add temporary ID or something? */
-            pilot_outfitLInitAll( eq_wgt.selected->p );
+         pilot_outfitLInitAll( eq_wgt.selected->p );
+         pilot_outfitLUpdate( player.p, 0. );
          pilot_calcStats( eq_wgt.selected->p ); /* TODO avoid running twice. */
       }
 
@@ -1414,7 +1434,8 @@ static int equipment_swapSlot( unsigned int wid, Pilot *p,
    pilot_weaponSafe( eq_wgt.selected->p );
 
    /* Notify GUI of modification. */
-   gui_setShip();
+   if ( ( p->id > 0 ) && pilot_isPlayer( p ) && ( p->ai != NULL ) )
+      gui_setShip();
 
    return 0;
 }
@@ -1589,7 +1610,8 @@ void equipment_addAmmo( void )
    pilot_fillAmmo( p );
 
    /* Notify GUI of modification. */
-   gui_setShip();
+   if ( ( p->id > 0 ) && pilot_isPlayer( p ) && ( p->ai != NULL ) )
+      gui_setShip();
 }
 
 /**
@@ -1598,7 +1620,8 @@ void equipment_addAmmo( void )
  *    @param buf Buffer to write to.
  *    @param max_len Maximum length of the string to allocate.
  *    @param s Pilot to get stats of.
- *    @param dpseps Whether or not to display dps and eps.
+ *    @param dpseps Whether or not to display damage per second and energy per
+ * second.
  *    @param name Whether or not to display the pilot name.
  */
 int equipment_shipStats( char *buf, int max_len, const Pilot *s, int dpseps,
@@ -1718,15 +1741,16 @@ static void equipment_genShipList( unsigned int wid )
    /* Add player's current ship. */
    cships[0].image   = ship_gfxStore( player.p->ship, 256, 0., 0., 0. );
    cships[0].caption = strdup( player.p->name );
-   cships[0].layers  = gl_copyTexArray( player.p->ship->gfx_overlays );
-   t                 = gl_newImage( OVERLAY_GFX_PATH "active.webp", 0 );
-   cships[0].layers  = gl_addTexArray( cships[0].layers, t );
+   cships[0].layers =
+      gl_copyTexArray( (const glTexture **)player.p->ship->gfx_overlays );
+   t                = gl_newImage( OVERLAY_GFX_PATH "active", 0 );
+   cships[0].layers = gl_addTexArray( cships[0].layers, t );
    if ( player.ps.favourite ) {
-      t                = gl_newImage( OVERLAY_GFX_PATH "favourite.webp", 0 );
+      t                = gl_newImage( OVERLAY_GFX_PATH "favourite", 0 );
       cships[0].layers = gl_addTexArray( cships[0].layers, t );
    }
    if ( player.p->ship->rarity > 0 ) {
-      snprintf( r, sizeof( r ), OVERLAY_GFX_PATH "rarity_%d.webp",
+      snprintf( r, sizeof( r ), OVERLAY_GFX_PATH "rarity_%d",
                 player.p->ship->rarity );
       t                = gl_newImage( r, 0 );
       cships[0].layers = gl_addTexArray( cships[0].layers, t );
@@ -1737,17 +1761,18 @@ static void equipment_genShipList( unsigned int wid )
       for ( int i = 1; i <= array_size( ps ); i++ ) {
          cships[i].image = ship_gfxStore( ps[i - 1].p->ship, 256, 0., 0., 0. );
          cships[i].caption = strdup( ps[i - 1].p->name );
-         cships[i].layers  = gl_copyTexArray( ps[i - 1].p->ship->gfx_overlays );
+         cships[i].layers  = gl_copyTexArray(
+            (const glTexture **)ps[i - 1].p->ship->gfx_overlays );
          if ( ps[i - 1].favourite ) {
-            t = gl_newImage( OVERLAY_GFX_PATH "favourite.webp", 0 );
+            t                = gl_newImage( OVERLAY_GFX_PATH "favourite", 0 );
             cships[i].layers = gl_addTexArray( cships[i].layers, t );
          }
          if ( ps[i - 1].deployed ) {
-            t                = gl_newImage( OVERLAY_GFX_PATH "fleet.webp", 0 );
+            t                = gl_newImage( OVERLAY_GFX_PATH "fleet", 0 );
             cships[i].layers = gl_addTexArray( cships[i].layers, t );
          }
          if ( ps[i - 1].p->ship->rarity > 0 ) {
-            snprintf( r, sizeof( r ), OVERLAY_GFX_PATH "rarity_%d.webp",
+            snprintf( r, sizeof( r ), OVERLAY_GFX_PATH "rarity_%d",
                       ps[i - 1].p->ship->rarity );
             t                = gl_newImage( r, 0 );
             cships[i].layers = gl_addTexArray( cships[i].layers, t );
@@ -1824,11 +1849,11 @@ static int equipment_filter( const Outfit *o )
       return 0;
 
    case 3:
-      return ( o->slot.size == OUTFIT_SLOT_SIZE_LIGHT );
+      return ( outfit_slotSize( o ) == OUTFIT_SLOT_SIZE_LIGHT );
    case 4:
-      return ( o->slot.size == OUTFIT_SLOT_SIZE_MEDIUM );
+      return ( outfit_slotSize( o ) == OUTFIT_SLOT_SIZE_MEDIUM );
    case 5:
-      return ( o->slot.size == OUTFIT_SLOT_SIZE_HEAVY );
+      return ( outfit_slotSize( o ) == OUTFIT_SLOT_SIZE_HEAVY );
    }
    return 1;
 }
@@ -2030,8 +2055,8 @@ void equipment_updateShips( unsigned int wid, const char *str )
    /* Just in case, turn off outfits and reset stats. */
    effect_clear( &ship->effects );
    pilot_outfitOffAll( ship );
-   if ( ship->id )
-      pilot_outfitLInitAll( ship );
+   pilot_outfitLInitAll( ship );
+   pilot_outfitLUpdate( player.p, 0. );
    pilot_calcStats( ship );
 
    /* Select. */
@@ -2047,7 +2072,7 @@ void equipment_updateShips( unsigned int wid, const char *str )
    spaceworthy =
       !pilot_reportSpaceworthy( ship, errorReport, sizeof( errorReport ) );
 
-   jumps = floor( ship->fuel_max / ship->fuel_consumption );
+   jumps = pilot_getJumps( ship );
 
    /* Get acquired text length. */
    x        = 20 + sw + 20 + 180 + 10;
@@ -2095,7 +2120,7 @@ void equipment_updateShips( unsigned int wid, const char *str )
       l += scnprintf( &buf[l], sizeof( buf ) - l, "\n%s", _( "Signature:" ) );
       l += scnprintf( &buf[l], sizeof( buf ) - l, "\n%s", _( "Stealth at:" ) );
       l +=
-         scnprintf( &buf[l], sizeof( buf ) - l, "\n%s", _( "Scanning time:" ) );
+         scnprintf( &buf[l], sizeof( buf ) - l, "\n%s", _( "Scanned time:" ) );
       l += scnprintf( &buf[l], sizeof( buf ) - l, "\n" );
       l += scnprintf( &buf[l], sizeof( buf ) - l, "\n%s", _( "Absorption:" ) );
       l += scnprintf( &buf[l], sizeof( buf ) - l, "\n%s", _( "Shield:" ) );
@@ -2231,8 +2256,9 @@ void equipment_updateShips( unsigned int wid, const char *str )
       for ( int i = 0; i < SHIP_CLASS_TOTAL; i++ )
          destroyed += ps->ships_destroyed[i];
       l += scnprintf( &buf[l], sizeof( buf ) - l, "\n" );
-      l += scnprintf( &buf[l], sizeof( buf ) - l, _( "%s hours" ),
-                      num2strU( ps->time_played / 3600., 1 ) );
+      l += scnprintf(
+         &buf[l], sizeof( buf ) - l, _( "%s hours" ),
+         num2strU( round( 10. * ps->time_played / 3600. ) / 10., 1 ) );
       l += scnprintf( &buf[l], sizeof( buf ) - l, "\n%s",
                       num2strU( ps->jumped_times, 0 ) );
       l += scnprintf( &buf[l], sizeof( buf ) - l, "\n%s",
@@ -2256,7 +2282,7 @@ void equipment_updateShips( unsigned int wid, const char *str )
       int             tx, ty, tw, th;
       ImageArrayCell *cells;
       int             ncells  = array_size( ship->outfit_intrinsic );
-      Outfit const  **outfits = (Outfit const **)array_create( Outfit  *);
+      Outfit const  **outfits = (Outfit const **)array_create( Outfit * );
       for ( int i = 0; i < ncells; i++ )
          array_push_back( &outfits, ship->outfit_intrinsic[i].outfit );
 
@@ -2599,7 +2625,8 @@ static void equipment_unequipShip( unsigned int wid, const char *str )
    pilot_weaponSafe( ship );
 
    /* Notify GUI of modification. */
-   gui_setShip();
+   if ( ( ship->id > 0 ) && pilot_isPlayer( ship ) && ( ship->ai != NULL ) )
+      gui_setShip();
 }
 
 /**
@@ -2624,7 +2651,7 @@ static void equipment_autoequipShip( unsigned int wid, const char *str )
    }
 
    /* Create the environment */
-   if ( autoequip_env == LUA_NOREF ) {
+   if ( autoequip_env == NULL ) {
       /* Read File. */
       size_t bufsize;
       char  *buf = ndata_read( file, &bufsize );
