@@ -10,7 +10,7 @@
 #include <assert.h>
 #include <stdlib.h>
 
-#include "SDL_timer.h"
+#include <SDL3/SDL_timer.h>
 
 #include "naev.h"
 /** @endcond */
@@ -94,16 +94,16 @@ typedef struct Faction_ {
    double override;   /**< Override the player standing. */
 
    /* Scheduler. */
-   nlua_env sched_env; /**< Lua scheduler script. */
+   nlua_env *sched_env; /**< Lua scheduler script. */
 
    /* Behaviour. */
-   double   friendly_at;        /**< Value of "standing.friendly_at" */
-   nlua_env lua_env;            /**< Faction specific environment. */
-   int      lua_hit;            /**< "standing.hit" */
-   int      lua_hit_test;       /**< "standing.hit_test" */
-   int      lua_text_rank;      /**< "standing.text_rank" */
-   int      lua_text_broad;     /**< "standing.text_broad" */
-   int      lua_reputation_max; /**< "standing.reputation_max" */
+   double    friendly_at;        /**< Value of "standing.friendly_at" */
+   nlua_env *lua_env;            /**< Faction specific environment. */
+   int       lua_hit;            /**< "standing.hit" */
+   int       lua_hit_test;       /**< "standing.hit_test" */
+   int       lua_text_rank;      /**< "standing.text_rank" */
+   int       lua_text_broad;     /**< "standing.text_broad" */
+   int       lua_reputation_max; /**< "standing.reputation_max" */
 
    /* Safe lanes. */
    double
@@ -116,7 +116,7 @@ typedef struct Faction_ {
       *generators; /**< Secondary factions generated by this faction. */
 
    /* Equipping. */
-   nlua_env equip_env; /**< Faction equipper enviornment. */
+   nlua_env *equip_env; /**< Faction equipper enviornment. */
 
    /* Flags. */
    unsigned int flags;  /**< Flags affecting the faction. */
@@ -170,22 +170,20 @@ static int faction_getRaw( const char *name )
    if ( strcmp( name, "Escort" ) == 0 )
       return FACTION_PLAYER;
 
-   if ( name != NULL ) {
-      int i;
-      for ( i = 0; i < array_size( faction_stack ); i++ )
-         if ( strcmp( faction_stack[i].name, name ) == 0 )
-            break;
+   int i;
+   for ( i = 0; i < array_size( faction_stack ); i++ )
+      if ( strcmp( faction_stack[i].name, name ) == 0 )
+         break;
 
-      if ( i != array_size( faction_stack ) )
-         return i;
+   if ( i != array_size( faction_stack ) )
+      return i;
 
-      /* Dynamic factions are why we can't have nice things.
+   /* Dynamic factions are why we can't have nice things.
       const Faction f = { .name = (char*)name };
       Faction *found = bsearch( &f, faction_stack, array_size(faction_stack),
       sizeof(Faction), faction_cmp ); if (found != NULL) return found -
       faction_stack;
       */
-   }
    return -1;
 }
 
@@ -434,7 +432,7 @@ const char *faction_default_ai( int f )
  * @brief Gets the tags the faction has.
  *
  *    @param f Faction ID.
- *    @return The tagss the faction has (array.h).
+ *    @return The tags the faction has (array.h).
  */
 const char *const *faction_tags( int f )
 {
@@ -606,7 +604,7 @@ void faction_addEnemy( int f, int o )
       return;
    }
 
-   /* player cannot be made an enemy this way */
+   /* Player cannot be made an enemy this way */
    if ( f == FACTION_PLAYER ) {
       WARN( _( "%d is the player faction" ), f );
       return;
@@ -752,14 +750,90 @@ void faction_rmAlly( int f, int o )
    }
 }
 
+/**
+ * @brief Adds an neutral to the faction's neutrals list.
+ *
+ *    @param f The faction to add an neutral to.
+ *    @param o The other faction to make an neutral.
+ */
+void faction_addNeutral( int f, int o )
+{
+   Faction *ff;
+   int     *tmp;
+
+   if ( f == o )
+      return;
+
+   if ( faction_isFaction( f ) )
+      ff = &faction_stack[f];
+   else { /* f is invalid */
+      WARN( _( "Faction id '%d' is invalid." ), f );
+      return;
+   }
+
+   if ( !faction_isFaction( o ) ) { /* o is invalid */
+      WARN( _( "Faction id '%d' is invalid." ), o );
+      return;
+   }
+
+   /* player cannot be made an neutral this way */
+   if ( f == FACTION_PLAYER ) {
+      WARN( _( "%d is the player faction" ), f );
+      return;
+   }
+   if ( o == FACTION_PLAYER ) {
+      WARN( _( "%d is the player faction" ), o );
+      return;
+   }
+
+   for ( int i = 0; i < array_size( ff->neutrals ); i++ ) {
+      if ( ff->neutrals[i] == o )
+         return;
+   }
+
+   tmp  = &array_grow( &ff->neutrals );
+   *tmp = o;
+
+   faction_computeGrid();
+}
+
+/**
+ * @brief Removes an neutral from the faction's neutrals list.
+ *
+ *    @param f The faction to remove an neutral from.
+ *    @param o The other faction to remove as an neutral.
+ */
+void faction_rmNeutral( int f, int o )
+{
+   Faction *ff;
+
+   if ( f == o )
+      return;
+
+   if ( faction_isFaction( f ) )
+      ff = &faction_stack[f];
+   else { /* f is invalid */
+      WARN( _( "Faction id '%d' is invalid." ), f );
+      return;
+   }
+
+   for ( int i = 0; i < array_size( ff->neutrals ); i++ ) {
+      if ( ff->neutrals[i] == o ) {
+         array_erase( &ff->neutrals, &ff->neutrals[i], &ff->neutrals[i + 1] );
+         faction_computeGrid();
+         return;
+      }
+   }
+}
+
 /**
  * @brief Gets the state associated to the faction scheduler.
  */
-nlua_env faction_getScheduler( int f )
+nlua_env *faction_getScheduler( int f )
 {
    if ( !faction_isFaction( f ) ) {
       WARN( _( "Faction id '%d' is invalid." ), f );
-      return LUA_NOREF;
+      return NULL;
    }
    return faction_stack[f].sched_env;
 }
@@ -767,11 +841,11 @@ nlua_env faction_getScheduler( int f )
 /**
  * @brief Gets the equipper state associated to the faction scheduler.
  */
-nlua_env faction_getEquipper( int f )
+nlua_env *faction_getEquipper( int f )
 {
    if ( !faction_isFaction( f ) ) {
       WARN( _( "Faction id '%d' is invalid." ), f );
-      return LUA_NOREF;
+      return NULL;
    }
    return faction_stack[f].equip_env;
 }
@@ -799,14 +873,18 @@ static double faction_hitLua( int f, const StarSystem *sys, double mod,
 
    Faction *faction = &faction_stack[f];
 
-   /* Make sure it's not static. */
-   if ( faction_isFlag( faction, FACTION_STATIC ) )
+   /* Make sure it's not static nor dynamic. */
+   if ( faction_isFlag( faction, FACTION_STATIC ) ||
+        faction_isFlag( faction, FACTION_DYNAMIC ) )
       return 0.;
 
    /* Overriden, so doesn't budge. */
    if ( faction_isFlag( faction, FACTION_REPOVERRIDE ) )
       return 0.;
 
+   if ( faction->lua_env == NULL )
+      return 0;
+
    /* Set up the function:
     * standing:hit( sys, amount, source, secondary ) */
    lua_rawgeti( naevL, LUA_REGISTRYINDEX, faction->lua_hit );
@@ -914,14 +992,18 @@ double faction_hitTest( int f, const StarSystem *sys, double mod,
 
    Faction *faction = &faction_stack[f];
 
-   /* Make sure it's not static. */
-   if ( faction_isFlag( faction, FACTION_STATIC ) )
+   /* Make sure it's not static nor dynamic. */
+   if ( faction_isFlag( faction, FACTION_STATIC ) ||
+        faction_isFlag( faction, FACTION_DYNAMIC ) )
       return 0.;
 
    /* Overriden, so doesn't budge. */
    if ( faction_isFlag( faction, FACTION_REPOVERRIDE ) )
       return 0.;
 
+   if ( faction->lua_env == NULL )
+      return 0;
+
    /* Set up the function:
     * standing:hit( sys, amount, source, secondary ) */
    lua_rawgeti( naevL, LUA_REGISTRYINDEX, faction->lua_hit );
@@ -960,9 +1042,9 @@ double faction_hitTest( int f, const StarSystem *sys, double mod,
  *    @param source Source of the faction modifier.
  *
  *   Possible sources:
- *    - "kill" : Pilot death.
- *    - "distress" : Pilot distress signal.
- *    - "script" : Either a mission or an event.
+ *    - `destroy` : Pilot death.
+ *    - `distress` : Pilot distress signal.
+ *    - `script` : Either a mission or an event.
  */
 void faction_modPlayer( int f, double mod, const char *source )
 {
@@ -998,9 +1080,9 @@ void faction_modPlayer( int f, double mod, const char *source )
  *    @param source Source of the faction modifier.
  *
  *   Possible sources:
- *    - "kill" : Pilot death.
- *    - "distress" : Pilot distress signal.
- *    - "script" : Either a mission or an event.
+ *    - `destroy` : Pilot death.
+ *    - `distress` : Pilot distress signal.
+ *    - `script` : Either a mission or an event.
  *
  * @sa faction_modPlayer
  */
@@ -1132,8 +1214,10 @@ double faction_reputation( int f )
       const Faction *fac = &faction_stack[f];
       if ( faction_isFlag( fac, FACTION_REPOVERRIDE ) )
          return fac->override;
-      else
-         return round( fac->player );
+      else {
+         return round( fac->player ) + 0.; // With IEC 60559 floating-point,
+                                           // this should convert -0. to +0.
+      }
    }
    WARN( _( "Faction id '%d' is invalid." ), f );
    return -1000.;
@@ -1219,7 +1303,7 @@ const glColour *faction_reputationColourSystem( int f, const StarSystem *sys )
 /**
  * @brief Gets the faction character associated to its standing with the player.
  *
- * Use this to do something like "#%c", faction_reputationColourChar(
+ * Use this to do something like `#%c`, faction_reputationColourChar(
  * some_faction ) in the font print routines.
  *
  *    @param f Faction to get the colour of based on player's standing.
@@ -1280,7 +1364,7 @@ const char *faction_getStandingTextAtValue( int f, double value )
 
    faction = &faction_stack[f];
 
-   if ( faction->lua_env == LUA_NOREF )
+   if ( faction->lua_env == NULL )
       return _( "???" );
    else {
       const char *r;
@@ -1334,7 +1418,7 @@ const char *faction_getStandingBroad( int f, int bribed, int override )
 
    faction = &faction_stack[f];
 
-   if ( faction->lua_env == LUA_NOREF )
+   if ( faction->lua_env == NULL )
       return _( "???" );
 
    /* Set up the method:
@@ -1386,7 +1470,7 @@ double faction_reputationMax( int f )
 
    faction = &faction_stack[f];
 
-   if ( faction->lua_env == LUA_NOREF )
+   if ( faction->lua_env == NULL )
       return 0.;
 
    /* Set up the method:
@@ -1568,9 +1652,9 @@ static int faction_parse( Faction *temp, const char *file )
    /* Clear memory. */
    memset( temp, 0, sizeof( Faction ) );
    temp->local_th           = 10.;
-   temp->equip_env          = LUA_NOREF;
-   temp->sched_env          = LUA_NOREF;
-   temp->lua_env            = LUA_NOREF;
+   temp->equip_env          = NULL;
+   temp->sched_env          = NULL;
+   temp->lua_env            = NULL;
    temp->lua_hit            = LUA_NOREF;
    temp->lua_hit_test       = LUA_NOREF;
    temp->lua_text_rank      = LUA_NOREF;
@@ -1636,7 +1720,7 @@ static int faction_parse( Faction *temp, const char *file )
             WARN( _( "Faction '%s' has duplicate 'logo' tag." ), temp->name );
             continue;
          }
-         snprintf( buf, sizeof( buf ), FACTION_LOGO_PATH "%s.webp",
+         snprintf( buf, sizeof( buf ), FACTION_LOGO_PATH "%s",
                    xml_get( node ) );
          temp->logo = gl_newImage( buf, 0 );
          continue;
@@ -1707,7 +1791,7 @@ static int faction_parse( Faction *temp, const char *file )
  * @brief Sets up a standing script for a faction.
  *
  *    @param temp Faction to associate the script to.
- *    @param scriptname Name of the lua script to use (e.g., "static").
+ *    @param scriptname Name of the Lua script to use (e.g., `static`).
  */
 static void faction_addStandingScript( Faction *temp, const char *scriptname )
 {
@@ -1725,7 +1809,7 @@ static void faction_addStandingScript( Faction *temp, const char *scriptname )
                "Most likely Lua file has improper syntax, please check" ),
             buf, lua_tostring( naevL, -1 ) );
       nlua_freeEnv( temp->lua_env );
-      temp->lua_env = LUA_NOREF;
+      temp->lua_env = NULL;
       free( dat );
       lua_pop( naevL, 1 );
       return;
@@ -1749,7 +1833,7 @@ static void faction_addStandingScript( Faction *temp, const char *scriptname )
 }
 
 /**
- * @brief Parses the social tidbits of a faction: allies and enemies.
+ * @brief Parses the social titbits of a faction: allies and enemies.
  *
  *    @param file File to parse.
  *    @return 0 on success.
@@ -1898,7 +1982,8 @@ void faction_updateSingle( int f )
          n++;
       }
    }
-   faction_stack[f].player = v / (double)n;
+   if ( n > 0 )
+      faction_stack[f].player = v / (double)n;
 }
 
 /**
@@ -1925,10 +2010,12 @@ void faction_applyLocalThreshold( int f, StarSystem *sys )
    }
 #endif /* DEBUGGING */
 
+   Faction    *fct       = &faction_stack[f];
    double      n         = 0;
    double      rep       = srep->local;
    StarSystem *sys_stack = system_getAll();
-   double      th        = faction_stack[f].local_th;
+   double      th        = fct->local_th;
+   int         usehidden = faction_isFlag( fct, FACTION_USESHIDDENJUMPS );
    /* TODO avoid a memory allocation every call. */
    int *done   = array_create( int );
    int *queuea = array_create( int );
@@ -1951,6 +2038,12 @@ void faction_applyLocalThreshold( int f, StarSystem *sys )
          for ( int j = 0; j < array_size( qsys->jumps ); j++ ) {
             StarSystem *nsys  = qsys->jumps[j].target;
             int         found = 0;
+
+            /* Don't propagate through hidden jumps if the faction doesn't use
+             * hidden. */
+            if ( jp_isFlag( &qsys->jumps[j], JP_HIDDEN ) && !usehidden )
+               continue;
+
             /* Ignore systems already looked at. */
             for ( int k = 0; k < array_size( done ); k++ ) {
                if ( nsys->id == done[k] ) {
@@ -2005,9 +2098,9 @@ int factions_load( void )
    memset( f, 0, sizeof( Faction ) );
    f->name               = strdup( "Player" );
    f->flags              = FACTION_STATIC | FACTION_INVISIBLE;
-   f->equip_env          = LUA_NOREF;
-   f->sched_env          = LUA_NOREF;
-   f->lua_env            = LUA_NOREF;
+   f->equip_env          = NULL;
+   f->sched_env          = NULL;
+   f->lua_env            = NULL;
    f->lua_hit            = LUA_NOREF;
    f->lua_hit_test       = LUA_NOREF;
    f->lua_text_rank      = LUA_NOREF;
@@ -2099,7 +2192,7 @@ int factions_load( void )
    return 0;
 }
 
-void factions_loadPost( void )
+int factions_loadPost( void )
 {
    for ( int i = 0; i < array_size( faction_stack ); i++ ) {
       Faction *f = &faction_stack[i];
@@ -2123,7 +2216,7 @@ void factions_loadPost( void )
                      "Most likely Lua file has improper syntax, please check" ),
                   buf, lua_tostring( naevL, -1 ) );
             nlua_freeEnv( f->sched_env );
-            f->sched_env = LUA_NOREF;
+            f->sched_env = NULL;
          }
          free( dat );
       }
@@ -2143,15 +2236,16 @@ void factions_loadPost( void )
                      "Most likely Lua file has improper syntax, please check" ),
                   buf, lua_tostring( naevL, -1 ) );
             nlua_freeEnv( f->equip_env );
-            f->equip_env = LUA_NOREF;
+            f->equip_env = NULL;
          }
          free( dat );
       }
 
       /* Make sure functions that need Lua have it. */
-      if ( ( f->lua_env == LUA_NOREF ) && !faction_isFlag( f, FACTION_STATIC ) )
+      if ( ( f->lua_env == NULL ) && !faction_isFlag( f, FACTION_STATIC ) )
          WARN( _( "Faction '%s' has no Lua and isn't static!" ), f->name );
    }
+   return 0;
 }
 
 /**
@@ -2404,9 +2498,9 @@ int faction_dynAdd( int base, const char *name, const char *display,
    f->ai          = ( ai == NULL ) ? NULL : strdup( ai );
    f->allies      = array_create( int );
    f->enemies     = array_create( int );
-   f->equip_env   = LUA_NOREF;
-   f->lua_env     = LUA_NOREF;
-   f->sched_env   = LUA_NOREF;
+   f->equip_env   = NULL;
+   f->lua_env     = NULL;
+   f->sched_env   = NULL;
    f->flags =
       FACTION_STATIC | FACTION_INVISIBLE | FACTION_DYNAMIC | FACTION_KNOWN;
    faction_addStandingScript( f, "static" );
