@@ -64,7 +64,7 @@ typedef struct InfoButton_s {
    char *button;   /**< Button widget name. */
    int   priority; /**< Button priority. */
    /* Lua stuff .*/
-   nlua_env    env;  /**< Runtime environment. */
+   nlua_env   *env;  /**< Runtime environment. */
    int         func; /**< Function to call. */
    SDL_Keycode key;  /**< Hotkey (or SDLK_UNKNOWN==0 if none). */
 } InfoButton_t;
@@ -426,8 +426,9 @@ static void info_openMain( unsigned int wid )
    l += scnprintf( &str[l], sizeof( str ) - l, "\n%d / %d %s", cargo_used,
                    cargo_total, UNIT_MASS );
    l += scnprintf( &str[l], sizeof( str ) - l, "%s", "\n\n" );
-   l += scnprintf( &str[l], sizeof( str ) - l, _( "%.1f hours" ),
-                   player.time_played / 3600. );
+   l += scnprintf(
+      &str[l], sizeof( str ) - l, _( "%s hours" ),
+      num2strU( round( 10. * player.time_played / 3600. ) / 10., 1 ) );
    l += scnprintf( &str[l], sizeof( str ) - l, "\n%s",
                    num2strU( (double)player.death_counter, 0 ) );
    l += scnprintf( &str[l], sizeof( str ) - l, "\n%s",
@@ -451,7 +452,7 @@ static void info_openMain( unsigned int wid )
 
    /* menu */
    window_addButtonKey( wid, -20, 20, BUTTON_WIDTH, BUTTON_HEIGHT, "btnClose",
-                        _( "Close" ), info_close, SDLK_c );
+                        _( "Close" ), info_close, SDLK_C );
 
    /* TODO probably add alt text with descriptions. */
    lic = player_getLicenses();
@@ -459,7 +460,7 @@ static void info_openMain( unsigned int wid )
    n   = array_size( lic ) + array_size( inv );
    /* List. */
    if ( n == 0 ) {
-      inventory    = malloc( sizeof( char    *) );
+      inventory    = malloc( sizeof( char * ) );
       inventory[0] = strdup( _( "None" ) );
       n            = 1;
    } else {
@@ -467,7 +468,7 @@ static void info_openMain( unsigned int wid )
       int ninv  = array_size( inv );
       inventory = malloc( sizeof( char * ) * n );
       for ( int i = 0; i < nlic; i++ )
-         SDL_asprintf( &inventory[i], "#n%s#0%s", _( "License: " ),
+         SDL_asprintf( &inventory[i], "#n%s#0%s", _( "Licence: " ),
                        _( lic[i] ) );
       qsort( inventory, nlic, sizeof( char * ), strsort );
       for ( int i = 0; i < ninv; i++ ) {
@@ -506,7 +507,7 @@ static void info_openShip( unsigned int wid )
 
    /* Buttons */
    window_addButtonKey( wid, -20, 20, BUTTON_WIDTH, BUTTON_HEIGHT,
-                        "closeOutfits", _( "Close" ), info_close, SDLK_c );
+                        "closeOutfits", _( "Close" ), info_close, SDLK_C );
 
    /* Text. */
    l += scnprintf( &buf[l], sizeof( buf ) - l, "%s", _( "Name:" ) );
@@ -654,15 +655,15 @@ static void info_openWeapons( unsigned int wid )
    if ( advanced ) {
       window_addButtonKey( wid, x + 10, y, BUTTON_WIDTH, BUTTON_HEIGHT,
                            "btnCycle", _( "Cycle Mode" ), weapons_toggleList,
-                           SDLK_m );
+                           SDLK_M );
       x += BUTTON_WIDTH + 10;
    }
    window_addButtonKey( wid, x + 10 + ( BUTTON_WIDTH + 10 ) * 0, y,
                         BUTTON_WIDTH, BUTTON_HEIGHT, "btnClear", _( "Clear" ),
-                        weapons_clear, SDLK_l );
+                        weapons_clear, SDLK_L );
    window_addButtonKey( wid, x + 10 + ( BUTTON_WIDTH + 10 ) * 1, y,
                         BUTTON_WIDTH, BUTTON_HEIGHT, "btnClearAll",
-                        _( "Clear All" ), weapons_clearAll, SDLK_a );
+                        _( "Clear All" ), weapons_clearAll, SDLK_A );
    x = 220;
    y -= BUTTON_HEIGHT + 10;
    if ( advanced ) {
@@ -703,13 +704,13 @@ static void info_openWeapons( unsigned int wid )
 
    /* Buttons */
    window_addButtonKey( wid, -20, 20, BUTTON_WIDTH, BUTTON_HEIGHT,
-                        "closeWeapons", _( "Close" ), info_close, SDLK_c );
+                        "closeWeapons", _( "Close" ), info_close, SDLK_C );
    window_addButtonKey( wid, -20 - 1 * ( BUTTON_WIDTH + 10 ), 20, BUTTON_WIDTH,
                         BUTTON_HEIGHT, "btnHelp", p_( "UI", "Help" ),
-                        weapons_help, SDLK_h );
+                        weapons_help, SDLK_H );
    window_addButtonKey( wid, -20 - 2 * ( BUTTON_WIDTH + 10 ), 20, BUTTON_WIDTH,
                         BUTTON_HEIGHT, "btnAdvanced", p_( "UI", "Advanced" ),
-                        weapons_advanced, SDLK_e );
+                        weapons_advanced, SDLK_E );
    if ( advanced )
       window_buttonCaption( wid, "btnAdvanced", p_( "UI", "Simple" ) );
 }
@@ -846,7 +847,7 @@ static void weapons_clearAll( unsigned int wid, const char *str )
 }
 
 /**
- * @brief Toggles autoweap for the ship.
+ * @brief Toggles automatic weapon groups for the ship.
  */
 static void weapons_autoweap( unsigned int wid, const char *str )
 {
@@ -931,18 +932,18 @@ static void weapons_renderLegend( double bx, double by, double bw, double bh,
    gl_print( &gl_defFont, bx, y, &cFontWhite, p_( "info", "Legend" ) );
 
    y -= 20.;
-   toolkit_drawRect( bx, y, 10, 10, &cFontBlue, NULL );
+   toolkit_drawRect( bx, y, 10, 10, &cFontBlue );
    gl_print( &gl_smallFont, bx + 20, y, &cFontWhite,
              _( "Outfit that can be activated" ) );
 
    y -= 20.;
-   toolkit_drawRect( bx, y, 10, 10, &cFontRed, NULL );
+   toolkit_drawRect( bx, y, 10, 10, &cFontRed );
    gl_print( &gl_smallFont, bx + 20, y, &cFontWhite,
              _( "Outfit that is enabled for the weapon set" ) );
 }
 
 /**
- * @brief Opens the naevpedia for weapon sets.
+ * @brief Opens Naevpedia for weapon sets.
  */
 static void weapons_help( unsigned int wid, const char *str )
 {
@@ -979,10 +980,10 @@ static void info_openCargo( unsigned int wid )
 
    /* Buttons */
    window_addButtonKey( wid, -20, 20, BUTTON_WIDTH, BUTTON_HEIGHT, "closeCargo",
-                        _( "Close" ), info_close, SDLK_c );
+                        _( "Close" ), info_close, SDLK_C );
    window_addButtonKey( wid, -20 - BUTTON_WIDTH - 10, 20, BUTTON_WIDTH,
                         BUTTON_HEIGHT, "btnJettisonCargo", _( "Jettison" ),
-                        cargo_jettison, SDLK_j );
+                        cargo_jettison, SDLK_J );
    window_disableButton( wid, "btnJettisonCargo" );
 
    /* Description. */
@@ -1012,7 +1013,7 @@ static void cargo_genList( unsigned int wid )
    /* List */
    if ( array_size( pclist ) == 0 ) {
       /* No cargo */
-      buf    = malloc( sizeof( char    *) );
+      buf    = malloc( sizeof( char * ) );
       buf[0] = strdup( _( "None" ) );
       nbuf   = 1;
    } else {
@@ -1023,9 +1024,14 @@ static void cargo_genList( unsigned int wid )
          int             misn = ( pc->id != 0 );
          int illegal          = ( array_size( pc->commodity->illegalto ) > 0 );
 
-         SDL_asprintf( &buf[i], "%s %d%s%s", _( pc->commodity->name ),
-                       pc->quantity, misn ? _( " [#bMission#0]" ) : "",
-                       illegal ? _( " (#rillegal#0)" ) : "" );
+         if ( pc->quantity > 0 )
+            SDL_asprintf( &buf[i], "%s %d%s%s", _( pc->commodity->name ),
+                          pc->quantity, misn ? _( " [#bMission#0]" ) : "",
+                          illegal ? _( " (#rillegal#0)" ) : "" );
+         else
+            SDL_asprintf( &buf[i], "%s%s%s", _( pc->commodity->name ),
+                          misn ? _( " [#bMission#0]" ) : "",
+                          illegal ? _( " (#rillegal#0)" ) : "" );
       }
       nbuf = array_size( pclist );
    }
@@ -1155,7 +1161,7 @@ static void cargo_jettison( unsigned int wid, const char *str )
       /* Now clean up mission. */
       if ( ret != 2 ) {
          mission_cleanup( misn );
-         mission_shift( pos );
+         mission_shift( f );
       }
 
       /* Reset markers. */
@@ -1166,9 +1172,17 @@ static void cargo_jettison( unsigned int wid, const char *str )
 
       /* Regenerate list. */
       mission_menu_genList( info_windows[INFO_WIN_MISN], 0 );
-   } else
+   } else {
+      double amount =
+         dialogue_fader( _( "Jettison Cargo?" ), 0., pclist[pos].quantity, 0.,
+                         _( "Jettison how many tonnes of %s?" ),
+                         _( pclist[pos].commodity->name ) );
+      int value = round( amount );
+      if ( value <= 0 )
+         return;
       /* Remove the cargo */
-      pfleet_cargoRm( pclist[pos].commodity, pclist[pos].quantity, 1 );
+      pfleet_cargoRm( pclist[pos].commodity, value, 1 );
+   }
 
    /* We reopen the menu to recreate the list now. */
    ship_update( info_windows[INFO_WIN_SHIP] );
@@ -1242,7 +1256,7 @@ static void info_openStandings( unsigned int wid )
 
    /* Buttons */
    window_addButtonKey( wid, -20, 20, BUTTON_WIDTH, BUTTON_HEIGHT,
-                        "closeMissions", _( "Close" ), info_close, SDLK_c );
+                        "closeMissions", _( "Close" ), info_close, SDLK_C );
 
    /* Graphics. */
    window_addImage( wid, 0, 0, 0, 0, "imgLogo", NULL, 0 );
@@ -1257,7 +1271,7 @@ static void info_openStandings( unsigned int wid )
 
    /* Gets the faction standings. */
    info_factions = faction_getKnown();
-   str           = malloc( sizeof( char           *) * array_size( info_factions ) );
+   str           = malloc( sizeof( char * ) * array_size( info_factions ) );
    qsort( info_factions, array_size( info_factions ), sizeof( int ),
           factionsSort );
 
@@ -1299,8 +1313,10 @@ static void standings_update( unsigned int wid, const char *str )
    y = -40;
    t = faction_logo( info_factions[p] );
    if ( t != NULL ) {
-      int tw = t->w * (double)FACTION_LOGO_SM / MAX( t->w, t->h );
-      int th = t->h * (double)FACTION_LOGO_SM / MAX( t->w, t->h );
+      int tw =
+         tex_w( t ) * (double)FACTION_LOGO_SM / MAX( tex_w( t ), tex_h( t ) );
+      int th =
+         tex_h( t ) * (double)FACTION_LOGO_SM / MAX( tex_w( t ), tex_h( t ) );
       window_modifyImage( wid, "imgLogo", t, tw, th );
       window_moveWidget( wid, "imgLogo", x + ( FACTION_LOGO_SM - tw ),
                          y - ( FACTION_LOGO_SM - th ) / 2 );
@@ -1391,15 +1407,14 @@ static void info_openMissions( unsigned int wid )
 
    /* buttons */
    window_addButtonKey( wid, -20, 20, BUTTON_WIDTH, BUTTON_HEIGHT,
-                        "closeMissions", _( "Close" ), info_close, SDLK_c );
+                        "closeMissions", _( "Close" ), info_close, SDLK_C );
    window_addButtonKey( wid, -20 - BUTTON_WIDTH - 10, 20, BUTTON_WIDTH,
                         BUTTON_HEIGHT, "btnAbortMission", _( "Abort" ),
-                        mission_menu_abort, SDLK_a );
+                        mission_menu_abort, SDLK_A );
 
    /* Add a checkbox to hide the mission. */
    window_addCheckbox( wid, 300 + 40, 20 + BUTTON_HEIGHT + 10, w - 300 - 60,
-                       BUTTON_HEIGHT, "chkHide",
-                       _( "Hide mission on-screen display" ),
+                       BUTTON_HEIGHT, "chkHide", _( "Hide mission" ),
                        mission_menu_chk_hide, 0 );
    /* Checkbox to make the mission be top. */
    window_addCheckbox(
@@ -1469,7 +1484,12 @@ static void mission_menu_update( unsigned int wid, const char *str )
    const StarSystem *sys;
    int               pos = toolkit_getListPos( wid, "lstMission" );
 
-   if ( pos < 0 || pos == selectedMission )
+   // Already selected
+   if ( pos == selectedMission )
+      return;
+
+   // Out of bounds.
+   if ( ( pos < 0 ) || ( pos >= array_size( player_missions ) ) )
       return;
 
    /* Modify the text. */
@@ -1498,19 +1518,22 @@ static void mission_menu_chk_hide( unsigned int wid, const char *str )
 {
    Mission *misn;
    int      pos = toolkit_getListPos( wid, "lstMission" );
-   if ( ( pos < 0 ) || ( pos > array_size( player_missions ) ) )
+   if ( ( pos < 0 ) || ( pos >= array_size( player_missions ) ) )
       return;
    misn = player_missions[pos];
 
    if ( misn->osd == 0 )
       return;
    misn_osdSetHide( misn, window_checkboxState( wid, str ) );
+
+   /* Update system markers. */
+   mission_sysMark();
 }
 static void mission_menu_chk_priority( unsigned int wid, const char *str )
 {
    Mission *misn;
    int      pos = toolkit_getListPos( wid, "lstMission" );
-   if ( ( pos < 0 ) || ( pos > array_size( player_missions ) ) )
+   if ( ( pos < 0 ) || ( pos >= array_size( player_missions ) ) )
       return;
    misn = player_missions[pos];
 
@@ -1624,7 +1647,7 @@ static void shiplog_menu_update( unsigned int wid, const char *str )
 }
 
 /**
- * @brief Translates a position in "lstLogType" to a shiplog "type" filter.
+ * @brief Translates a position in `lstLogType` to a shiplog `type` filter.
  */
 static const char *info_getLogTypeFilter( int lstPos )
 {
@@ -1787,7 +1810,7 @@ static void info_shiplogAdd( unsigned int wid, const char *str )
 static void info_openShipLog( unsigned int wid )
 {
    int w, h, texth;
-   /* re-initialise the statics */
+   /* re-initialize the statics */
    selectedLog     = 0;
    selectedLogType = 0;
 
@@ -1795,16 +1818,16 @@ static void info_openShipLog( unsigned int wid )
    window_dimWindow( wid, &w, &h );
    /* buttons */
    window_addButtonKey( wid, -20, 20, BUTTON_WIDTH, BUTTON_HEIGHT,
-                        "closeShipLog", _( "Close" ), info_close, SDLK_c );
+                        "closeShipLog", _( "Close" ), info_close, SDLK_C );
    window_addButtonKey( wid, -20 - 1 * ( 20 + BUTTON_WIDTH ), 20, BUTTON_WIDTH,
                         BUTTON_HEIGHT, "btnDeleteLog", _( "Delete" ),
-                        info_shiplogMenuDelete, SDLK_d );
+                        info_shiplogMenuDelete, SDLK_D );
    window_addButtonKey( wid, -20 - 2 * ( 20 + BUTTON_WIDTH ), 20, BUTTON_WIDTH,
                         BUTTON_HEIGHT, "btnViewLog", _( "View Entry" ),
-                        info_shiplogView, SDLK_v );
+                        info_shiplogView, SDLK_V );
    window_addButtonKey( wid, -20 - 3 * ( 20 + BUTTON_WIDTH ), 20, BUTTON_WIDTH,
                         BUTTON_HEIGHT, "btnAddLog", _( "Add Entry" ),
-                        info_shiplogAdd, SDLK_a );
+                        info_shiplogAdd, SDLK_A );
    /* Description text */
    texth = gl_printHeightRaw( &gl_smallFont, w, "Select log type" );
    window_addText( wid, 20, 80 + BUTTON_HEIGHT + LOGSPACING, w - 40, texth, 0,
