@@ -127,16 +127,16 @@ void shipyard_open( unsigned int wid )
    /* buttons */
    off = -20;
    window_addButtonKey( wid, off, 20, bw, bh, "btnCloseShipyard",
-                        _( "Take Off" ), land_buttonTakeoff, SDLK_t );
+                        _( "Take Off" ), land_buttonTakeoff, SDLK_T );
    off -= 20 + bw;
    window_addButtonKey( wid, off, 20, bw, bh, "btnTradeShip", _( "Trade-In" ),
-                        shipyard_trade, SDLK_r );
+                        shipyard_trade, SDLK_R );
    off -= 20 + bw;
    window_addButtonKey( wid, off, 20, bw, bh, "btnBuyShip", _( "Buy" ),
-                        shipyard_buy, SDLK_b );
+                        shipyard_buy, SDLK_B );
    off -= 20 + bw;
    window_addButtonKey( wid, off, 20, bw, bh, "btnNaevpediaOutfits",
-                        _( "Archives" ), shipyard_naevpedia, SDLK_a );
+                        _( "Archives" ), shipyard_naevpedia, SDLK_A );
 
    /* ship review */
    window_addRect( wid, -40 + 4, -40 + 4, sw + 8, sh + 8, "rctTarget", &cBlack,
@@ -197,7 +197,8 @@ void shipyard_open( unsigned int wid )
       for ( int i = 0; i < nships; i++ ) {
          cships[i].caption = strdup( _( shipyard_list[i]->name ) );
          cships[i].image   = ship_gfxStore( shipyard_list[i], 256, 0., 0., 0. );
-         cships[i].layers  = gl_copyTexArray( shipyard_list[i]->gfx_overlays );
+         cships[i].layers  = gl_copyTexArray(
+            (const glTexture **)shipyard_list[i]->gfx_overlays );
          if ( shipyard_list[i]->rarity > 0 ) {
             glTexture *t     = rarity_texture( shipyard_list[i]->rarity );
             cships[i].layers = gl_addTexArray( cships[i].layers, t );
@@ -375,7 +376,7 @@ void shipyard_update( unsigned int wid, const char *str )
    if ( ship->license ) {
       int meets_reqs = player_hasLicense( ship->license );
       ;
-      k += scnprintf( &lbl[k], sizeof( lbl ) - k, "\n%s", _( "License:" ) );
+      k += scnprintf( &lbl[k], sizeof( lbl ) - k, "\n%s", _( "Licence:" ) );
       if ( blackmarket )
          l += scnprintf( &buf[l], sizeof( buf ) - l, "\n%s#0",
                          _( "Not Necessary (Blackmarket)" ) );
@@ -533,7 +534,7 @@ static int shipyard_canAcquire( const Ship *ship, const Spob *spob,
 
    /* Must have the necessary license. */
    if ( !blackmarket && !player_hasLicense( ship->license ) ) {
-      land_errDialogueBuild( _( "You need the '%s' license to buy this ship." ),
+      land_errDialogueBuild( _( "You need the '%s' licence to buy this ship." ),
                              _( ship->license ) );
       failure = 1;
    }
@@ -756,23 +757,20 @@ static void shipyard_renderSlotsRow( double bx, double by, double bw,
          continue;
 
       /* Get the colour. */
-      c = outfit_slotSizeColour( &s[i].slot );
+      c = outfit_slotSizeColour( s[i].slot.size );
       if ( c == NULL )
          c = &cBlack;
 
       x += size + 7.;
-      toolkit_drawRect( x, by, size, size, c, NULL );
+      toolkit_drawRect( x, by, size, size, c );
 
       /* Add colour stripe depending on required/exclusiveness. */
       if ( s[i].required )
-         toolkit_drawTriangle( x, by, x + size, by + size, x, by + size,
-                               &cBrightRed );
+         toolkit_drawRectHalf( x, by, size, size, &cBrightRed );
       else if ( s[i].exclusive )
-         toolkit_drawTriangle( x, by, x + size, by + size, x, by + size,
-                               &cWhite );
+         toolkit_drawRectHalf( x, by, size, size, &cWhite );
       else if ( s[i].slot.spid != 0 )
-         toolkit_drawTriangle( x, by, x + size, by + size, x, by + size,
-                               &cBlack );
+         toolkit_drawRectHalf( x, by, size, size, &cBlack );
 
       gl_renderRectEmpty( x, by, size, size, &cBlack );
 
@@ -783,7 +781,7 @@ static void shipyard_renderSlotsRow( double bx, double by, double bw,
          double sh = 12.;
          double sx = x + 6;
          double sy = by + 6;
-         if ( icon->flags & OPENGL_TEX_SDF )
+         if ( tex_isSDF( icon ) )
             gl_renderSDF( icon, sx, sy, sw, sh, &cWhite, 0., 1. );
          else
             gl_renderScaleAspect( icon, sx, sy, sw, sh, NULL );
@@ -810,7 +808,7 @@ static int shipyard_mouseSlots( unsigned int wid, const SDL_Event *event,
       return 0;
 
    /* Only care about motion. */
-   if ( event->type != SDL_MOUSEMOTION )
+   if ( event->type != SDL_EVENT_MOUSE_MOTION )
       return 0;
 
    /* Find what row. */
@@ -858,15 +856,22 @@ static void shipyard_renderSlotsOver( double bx, double by, double bw,
    slot = wgt->slot;
    pos  = 0;
    if ( slot->slot.spid ) {
-      pos = scnprintf( alt, sizeof( alt ), "#o%s\n",
-                       _( sp_display( slot->slot.spid ) ) );
+      pos = scnprintf(
+         alt, sizeof( alt ), "#o%s\n",
+         pgettext_var( "slotproperty", sp_display( slot->slot.spid ) ) );
    } else
       pos = 0;
    pos += scnprintf( &alt[pos], sizeof( alt ) - pos, _( "#%c%s #%c%s #0slot" ),
-                     outfit_slotSizeColourFont( &slot->slot ),
+                     outfit_slotSizeColourFont( slot->slot.size ),
                      _( slotSize( slot->slot.size ) ),
-                     outfit_slotTypeColourFont( &slot->slot ),
+                     outfit_slotTypeColourFont( slot->slot.type ),
                      _( slotName( slot->slot.type ) ) );
+   if ( slot->exclusive && ( pos < (int)sizeof( alt ) ) )
+      pos += scnprintf( &alt[pos], sizeof( alt ) - pos, "#o%s#0",
+                        _( "\n[exclusive]" ) );
+   if ( slot->locked && ( pos < (int)sizeof( alt ) ) )
+      pos += scnprintf( &alt[pos], sizeof( alt ) - pos, "#r%s#0",
+                        _( "\n[locked]" ) );
 
    /* Draw the alt stuff. */
    toolkit_drawAltText( bx + wgt->altx, by + wgt->alty, alt );
@@ -891,8 +896,7 @@ static void preview_render( double x, double y, double w, double h, void *data )
       ship_renderGfxStore( p->fbo, shipyard_selected, SHIP_GFX_SIZE, p->dir,
                            p->updown, 0. );
 
-   gl_renderTextureRaw( p->tex, OPENGL_TEX_VFLIP, x, y, w, h, 0., 0., 1., 1.,
-                        NULL, 0. );
+   gl_renderTextureRaw( p->tex, 0, 0, x, y, w, h, 0., 0., 1., 1., NULL, 0. );
 }
 
 static int preview_mouse( unsigned int wid, const SDL_Event *event, double mx,
@@ -906,17 +910,17 @@ static int preview_mouse( unsigned int wid, const SDL_Event *event, double mx,
       return 0;
 
    switch ( event->type ) {
-   case SDL_MOUSEBUTTONUP:
+   case SDL_EVENT_MOUSE_BUTTON_UP:
       p->mousedown = 0;
       return 0;
 
-   case SDL_MOUSEBUTTONDOWN:
+   case SDL_EVENT_MOUSE_BUTTON_DOWN:
       if ( ( mx < 0. ) || ( mx > w ) || ( my < 0. ) || ( my > h ) )
          return 0;
       p->mousedown = 1;
       return 1;
 
-   case SDL_MOUSEMOTION:
+   case SDL_EVENT_MOUSE_MOTION:
       if ( !p->mousedown )
          return 0;
       p->dir += rx / ( SHIP_GFX_SIZE / M_PI * 0.5 );
