@@ -114,7 +114,7 @@ static const luaL_Reg shipL_methods[] = {
  *    @param env Environment to load ship library into.
  *    @return 0 on success.
  */
-int nlua_loadShip( nlua_env env )
+int nlua_loadShip( nlua_env *env )
 {
    nlua_register( env, SHIP_METATABLE, shipL_methods, 1 );
    return 0;
@@ -193,7 +193,7 @@ const Ship *luaL_validship( lua_State *L, int ind )
 const Ship **lua_pushship( lua_State *L, const Ship *ship )
 {
    const Ship **p;
-   p  = (const Ship **)lua_newuserdata( L, sizeof( Ship  *) );
+   p  = (const Ship **)lua_newuserdata( L, sizeof( Ship * ) );
    *p = ship;
    luaL_getmetatable( L, SHIP_METATABLE );
    lua_setmetatable( L, -2 );
@@ -340,7 +340,7 @@ static int shipL_baseType( lua_State *L )
  * @brief Gets the ship it inherits stats from if applicable.
  *
  *    @luatparam Ship s Ship to get the ship it is inheriting from.
- *    @luatreturn Ship|nil The ship it is inheritng from or nil if not
+ *    @luatreturn Ship|nil The ship it is inheriting from or nil if not
  * applicable.
  * @luafunc inherits
  */
@@ -490,13 +490,13 @@ static int shipL_fuelConsumption( lua_State *L )
 }
 
 /**
- * @brief Gets license required for the ship.
+ * @brief Gets licence required for the ship.
  *
- * @usage license = s:license()
+ * @usage licence = s:licence()
  *
- *    @luatparam Ship s Ship to get license of.
- *    @luatreturn string Required license or nil if not necessary.
- * @luafunc license
+ *    @luatparam Ship s Ship to get licence of.
+ *    @luatreturn string Required licence or nil if not necessary.
+ * @luafunc licence
  */
 static int shipL_license( lua_State *L )
 {
@@ -555,9 +555,9 @@ static int shipL_slots( lua_State *L )
  *    @luatparam Ship s Ship to get slots of
  *    @luatparam[opt=false] boolean ignore_locked Whether or not to ignore
  * locked slots.
- *    @luareturn A table of tables with slot properties string "size", string
- * "type", string "property", boolean "required", boolean "exclusive", boolean
- * "locked", and (if applicable) outfit "outfit" (Strings are English.)
+ *    @luareturn A table of tables with slot properties string `size`, string
+ * `type`, string `property`, boolean `required`, boolean `exclusive`, boolean
+ * `locked`, and (if applicable) outfit `outfit` (Strings are English.)
  * @luafunc getSlots
  */
 static int shipL_getSlots( lua_State *L )
@@ -589,38 +589,38 @@ static int shipL_getSlots( lua_State *L )
          lua_rawset(L, -3); /* table[key] = value*/
 #endif
 
-         lua_pushstring( L, "type" );                 /* key */
          lua_pushstring( L, slotName( slot->type ) ); /* value */
-         lua_rawset( L, -3 );                         /* table[key] = value*/
+         lua_setfield( L, -2, "type" );               /* table[key] = value*/
 
-         lua_pushstring( L, "size" ); /* key */
          lua_pushstring( L, slotSize( slot->size ) );
-         lua_rawset( L, -3 ); /* table[key] = value */
+         lua_setfield( L, -2, "size" ); /* table[key] = value */
 
-         lua_pushstring( L, "property" );               /* key */
          lua_pushstring( L, sp_display( slot->spid ) ); /* value */
-         lua_rawset( L, -3 );                           /* table[key] = value */
+         lua_setfield( L, -2, "property" );             /* table[key] = value */
 
-         lua_pushstring( L, "required" );       /* key */
          lua_pushboolean( L, sslot->required ); /* value */
-         lua_rawset( L, -3 );                   /* table[key] = value */
+         lua_setfield( L, -2, "required" );     /* table[key] = value */
 
-         lua_pushstring( L, "exclusive" );       /* key */
          lua_pushboolean( L, sslot->exclusive ); /* value */
-         lua_rawset( L, -3 );                    /* table[key] = value */
+         lua_setfield( L, -2, "exclusive" );     /* table[key] = value */
 
-         lua_pushstring( L, "locked" );       /* key */
          lua_pushboolean( L, sslot->locked ); /* value */
-         lua_rawset( L, -3 );                 /* table[key] = value */
+         lua_setfield( L, -2, "locked" );     /* table[key] = value */
 
-         lua_pushstring( L, "visible" );       /* key */
          lua_pushboolean( L, sslot->visible ); /* value */
-         lua_rawset( L, -3 );                  /* table[key] = value */
+         lua_setfield( L, -2, "visible" );     /* table[key] = value */
+
+         lua_newtable( L );
+         const char **tags = sp_tags( slot->spid );
+         for ( int t = 0; t < array_size( tags ); t++ ) {
+            lua_pushboolean( L, 1 );
+            lua_setfield( L, -2, tags[t] );
+         }
+         lua_setfield( L, -2, "tags" );
 
          if ( sslot->data != NULL ) {
-            lua_pushstring( L, "outfit" );    /* key */
             lua_pushoutfit( L, sslot->data ); /* value*/
-            lua_rawset( L, -3 );              /* table[key] = value */
+            lua_setfield( L, -2, "outfit" );  /* table[key] = value */
          }
 
          lua_rawseti( L, -2, k++ ); /* put the slot table in */
@@ -654,9 +654,10 @@ static const ShipOutfitSlot *ship_outfitSlotFromID( const Ship *s, int id )
  * @brief Checks to see if an outfit fits a ship slot.
  *
  *    @luatparam Ship s Ship to check.
- *    @luatparam number id ID of the slot to check (index in getSlots table).
+ *    @luatparam number id ID of the slot to check (index in `getSlots()`
+ * table).
  *    @luatparam Outfit o Outfit to check to see if it fits in the slot.
- *    @luatreturn boolean WHether or not the outfit fits the slot.
+ *    @luatreturn boolean Whether or not the outfit fits the slot.
  * @luafunc fitsSlot
  */
 static int shipL_fitsSlot( lua_State *L )
@@ -740,9 +741,9 @@ static int shipL_getSize( lua_State *L )
  * @brief Gets the path where the ship's graphics are located. Useful for seeing
  * if two ships share the same graphics.
  *
- *    @luatparam Ship s Ship to get the path of the graphis of.
+ *    @luatparam Ship s Ship to get the path of the graphics of.
  *    @luatreturn string The path to the ship graphics.
- * @luafunc size
+ * @luafunc gfxPath
  */
 static int shipL_gfxPath( lua_State *L )
 {
